/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0-german July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * Translation by Torsten Krueger (ToK).
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "Dialogs.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Text.h"


// ------------------------------------------------------------------
// Globals declared elsewhere
// ------------------------------------------------------------------
extern AppState GState;

// ------------------------------------------------------------------
// Globals used by "Edit teams"
// ------------------------------------------------------------------
Char*     GDialogTeamNames[MAX_TEAMS];
MemHandle GTeamNameHandle;
MemHandle GTeamPointsHandle;
// ------------------------------------------------------------------
// Globals used by "Edit fixtures"
// ------------------------------------------------------------------
Int8     GRound;
DateType GDate;
Char*    GMatchStrings[MAX_TEAMS];


// ------------------------------------------------------------------
//
// Edit teams
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Initializes "Edit team" form.
// ------------------------------------------------------------------
void InitEditTeamsForm() {
  Int8    i;
  ListPtr teamList;
  League  league = LeagueGetCurrent();
  Int16   sortedIndex[MAX_TEAMS];

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);

  // create an array of pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    GDialogTeamNames[i] = LeagueGetTeam(&league, sortedIndex[i]);
  }

  // give the pointer array to the list-control
  teamList = (ListPtr)UIGetObjectPtr(listID_teams);
  LstSetListChoices(teamList, GDialogTeamNames, league.teamCount);
  LstSetSelection(teamList, -1);
  // create and set handles for the team name and team points fields
  GTeamNameHandle = MemHandleNew(11);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teamname), GTeamNameHandle);
  GTeamPointsHandle = MemHandleNew(4);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teampoints), GTeamPointsHandle);
}


// ------------------------------------------------------------------
// Frees "Edit team" form.
// ------------------------------------------------------------------
void FreeEditTeamsForm() {
  Int8   i;
  League league = LeagueGetCurrent();

  // free the handles used by team name and team points fields
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teamname), NULL);
  MemHandleFree(GTeamNameHandle);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_teampoints), NULL);
  MemHandleFree(GTeamPointsHandle);

  // unlock the pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    MemPtrUnlock(GDialogTeamNames[i]);
  }
}


// ------------------------------------------------------------------
// Copies the selected team name to the team name text field.
// Also read the team points from the db and displays them.
// ------------------------------------------------------------------
void SelectTeam(Int16 listIndex) {
  Char    buf[4];
  Int16   teamIndex;
  Int16   sortedIndex[MAX_TEAMS];
  League  league = LeagueGetCurrent();

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);
  teamIndex = sortedIndex[listIndex];
  // get the points to a char buffer
  StrIToA(buf, LeagueGetTeamPoints(&league, teamIndex));

  // update the fieds
  UISetFieldText(fieldID_teamname, GDialogTeamNames[listIndex], true);
  UISetFieldText(fieldID_teampoints, buf, true);
}


// ------------------------------------------------------------------
// Saves the team name and the team points.
// ------------------------------------------------------------------
void SaveTeamData() {
  ListPtr teamList  = (ListPtr)UIGetObjectPtr(listID_teams);
  Char*   teamName  = FldGetTextPtr((FieldPtr)UIGetObjectPtr(fieldID_teamname));
  League  league    = LeagueGetCurrent();
  Int16   sortedIndex[MAX_TEAMS];
  Int16   teamIndex;
  Int8    points;
  Int16   selectedIndex = LstGetSelection(teamList);

  if (selectedIndex != noListSelection) {

    if (StrLen(teamName) == 0) {
      FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_TEAMS_MUST_HAVE_NAME, NULL, NULL);
      return;
    }

    // get indexes for the teams, sorted alphabetically
    LeagueGetSortedTeamIndexes(&league, sortedIndex);
    teamIndex = sortedIndex[selectedIndex];
    // save the new name
    LeagueSaveTeam(&league, teamIndex, teamName);
    // save the points
    points = (Int8)UIGetFieldIntValue(fieldID_teampoints);
    LeagueSaveTeamPoints(&league, teamIndex, points);
  }

  // clear the text fields and set list selection to none
  UISetFieldText(fieldID_teamname, "", true);
  UISetFieldText(fieldID_teampoints, "", true);
  LstSetSelection(teamList, -1);
  // free and re-init form so teams are re-sorted
  FreeEditTeamsForm();
  InitEditTeamsForm();
  // re-draw the list
  LstEraseList(teamList);
  LstDrawList(teamList);
}


// ------------------------------------------------------------------
// Eventhandler for "Edit teams" form.
// ------------------------------------------------------------------
Boolean EditTeamsFormEvtHandler(EventPtr event) {
  Boolean handled = false;

  switch (event->eType) {

    // form is opened
    case frmOpenEvent:
      InitEditTeamsForm();
      FrmDrawForm(FrmGetActiveForm());
      // init the text fields
      UISetFieldText(fieldID_teamname, "", true);
      UISetFieldText(fieldID_teampoints, "", true);
      handled = true;
      break;

    // form is closed
    case frmCloseEvent:
      FreeEditTeamsForm();
      break;

    // a team was tapped in the list
    case lstSelectEvent:
      SelectTeam(event->data.lstSelect.selection);
      handled = true;
      break;

    // a button was pressed
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {

        // close the form
        case btnID_close:
          FreeEditTeamsForm();
          FrmReturnToForm(0);
          ShowLeague(GState.currentLeagueNameIndex, GState.currentSeasonIndex, GState.mode, true, false);
          handled = true;
          break;

        // save the team name
        case btnID_save:
          SaveTeamData();
          handled = true;
          break;
      }
      break;

    default:
  }

  return handled;
}


// ------------------------------------------------------------------
//
// Edit fixtures
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Initializes "Edit fixtures" form.
// ------------------------------------------------------------------
void InitEditFixturesForm() {
  Int8    i;
  Char*   tempPtr;
  ListPtr hometeamList;
  ListPtr awayteamList;
  League  league = LeagueGetCurrent();
  Int16   sortedIndex[MAX_TEAMS];

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);

  // create an array of pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    tempPtr = LeagueGetTeam(&league, sortedIndex[i]);
    GDialogTeamNames[i] = MemPtrNew(11);
    StrCopy(GDialogTeamNames[i], tempPtr);
    MemPtrUnlock(tempPtr);
    GMatchStrings[i] = MemPtrNew(35);
  }
  // give the pointer array to the list-controls
  hometeamList = (ListPtr)UIGetObjectPtr(popListID_hometeam);
  LstSetListChoices(hometeamList, GDialogTeamNames, league.teamCount);
  UISelectDropDown(popListID_hometeam, popTrigID_hometeam, 0);
  LstSetHeight(hometeamList, min(league.teamCount, 10));
  awayteamList = (ListPtr)UIGetObjectPtr(popListID_awayteam);
  LstSetListChoices(awayteamList, GDialogTeamNames, league.teamCount);
  UISelectDropDown(popListID_awayteam, popTrigID_awayteam, 1);
  LstSetHeight(awayteamList, min(league.teamCount, 10));

  // set global variables
  GRound = league.roundCount;
  DateSecondsToDate(TimGetSeconds(), &GDate);
}


// ------------------------------------------------------------------
// Frees "Edit fixtures" form.
// ------------------------------------------------------------------
void FreeEditFixturesForm() {
  Int8   i;
  League league = LeagueGetCurrent();

  // unlock the pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    MemPtrFree(GMatchStrings[i]);
    MemPtrFree(GDialogTeamNames[i]);
  }
}


// ------------------------------------------------------------------
// Redraws the form
// ------------------------------------------------------------------
void UpdateEditFixturesForm() {
  Int16   i;
  Char    buf[20];
  FormPtr form = FrmGetActiveForm();
  ListPtr matchList;
  Int16   matchesInRound[MAX_TEAMS];
  Int8    matchesInRoundCount;
  League  league = LeagueGetCurrent();

  // get all the matches for the current round
  matchesInRoundCount = LeagueGetMatchesInRound(&league, GRound, matchesInRound);
  for(i = 0; i < matchesInRoundCount; i++) {
    LeagueGetMatchString(&league, matchesInRound[i], GMatchStrings[i]);
  }

  // set matches for the list control
  matchList = (ListPtr)UIGetObjectPtr(listID_fixtures);
  LstSetListChoices(matchList, GMatchStrings, matchesInRoundCount);
  LstSetSelection(matchList, -1);

  // round label
  StrPrintF(buf, TEXT_MATCH_ROUND, GRound);
  FrmHideObject(form, FrmGetObjectIndex(form, labelID_round));
  FrmCopyLabel(form, labelID_round, buf);
  FrmShowObject(form, FrmGetObjectIndex(form, labelID_round));

  // date control
  StrPrintF(buf, TEXT_EDIT_FIX_DATE, GDate.day, GDate.month);
  UISetControlLabel(form, selectTrigID_date, buf);

  // set values in drop-downs (because of plamos5 bug?)
  UISelectDropDown(popListID_hometeam, popTrigID_hometeam, LstGetSelection((ListPtr)UIGetObjectPtr(popListID_hometeam)));
  UISelectDropDown(popListID_awayteam, popTrigID_awayteam, LstGetSelection((ListPtr)UIGetObjectPtr(popListID_awayteam)));
}


// ------------------------------------------------------------------
// Called when 'Add' button is clicked
// ------------------------------------------------------------------
void AddMatch() {
  Match   match;
  ListPtr matchList = (ListPtr)UIGetObjectPtr(listID_fixtures);
  League  league = LeagueGetCurrent();
  Int16   sortedIndex[MAX_TEAMS];

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, sortedIndex);

  // check that we don't add too many matches
  if(LstGetNumberOfItems(matchList) >= league.teamCount) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_CANNOT_ADD_MATCH, NULL, NULL);
  }

  else {
    // get data from form
    match.round     = GRound;
    match.dateYear  = GDate.year + 1904; //GYear;
    match.dateMonth = GDate.month; //GMonth;
    match.dateDay   = GDate.day; //GDay;
    match.homeTeam  = sortedIndex[LstGetSelection((ListPtr)UIGetObjectPtr(popListID_hometeam))];
    match.awayTeam  = sortedIndex[LstGetSelection((ListPtr)UIGetObjectPtr(popListID_awayteam))];
    match.homeGoals = -1;
    match.awayGoals = -1;
    match.isPlayed  = false;

    // create the new match
    if(match.homeTeam != match.awayTeam) {
      LeagueAddMatch(&league, &match, -1);
    }
  }

}


// ------------------------------------------------------------------
// Called when 'Delete match' is selected
// ------------------------------------------------------------------
void DeleteMatch() {
  Int16  selectedIndex;
  Int16  matchesInRound[MAX_TEAMS];
  League league = LeagueGetCurrent();

  LeagueGetMatchesInRound(&league, GRound, matchesInRound);
  selectedIndex = LstGetSelection((ListPtr)UIGetObjectPtr(listID_fixtures));

  if(selectedIndex == noListSelection) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_NO_MATCH_SELECTED, NULL, NULL);
  }
  else {
    LeagueDeleteMatch(&league, GRound, matchesInRound[selectedIndex]);
  }
}


// ------------------------------------------------------------------
// Called when 'Delete round' is selected
// ------------------------------------------------------------------
void DeleteRound() {
  Int16  matchesInRound[MAX_TEAMS];
  Int8   matchesInRoundCount;
  League league = LeagueGetCurrent();

  matchesInRoundCount = LeagueGetMatchesInRound(&league, GRound, matchesInRound);

  if(GRound < league.roundCount) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_ONLY_DELETE_LAST_ROUND, NULL, NULL);
  }
  else if(GRound == 1) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_CANNOT_DELETE_FIRST_ROUND, NULL, NULL);
  }
  else if(matchesInRoundCount > 0) {
    FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_FIX_ONLY_DELETE_EMPTY_ROUND, NULL, NULL);
  }
  else {
    LeagueDeleteRound(&league, GRound);
    GRound--;
  }

}


// ------------------------------------------------------------------
// Eventhandler for "Edit fixtures" form.
// ------------------------------------------------------------------
Boolean EditFixturesFormEvtHandler(EventPtr event) {
  Boolean handled      = false;
  Boolean updateScreen = false;
  League  league;
  Int16   year, month, day;

  switch (event->eType) {

    // form is opened
    case frmOpenEvent:
      InitEditFixturesForm();
      updateScreen = true;
      handled = true;
      break;

   // form is closed
    case frmCloseEvent:
      FreeEditFixturesForm();
      break;

   // form need to be updated
    case frmUpdateEvent:
      updateScreen = true;
      handled = true;
      break;

   // one of the scrollbuttons was pressed
    case keyDownEvent:
      switch(event->data.keyDown.chr) {
        case pageUpChr :
          GRound = max(GRound-1, 1);
          updateScreen = true;
          handled = true;
          break;
        case pageDownChr:
          league = LeagueGetCurrent();
          GRound = min(GRound+1, league.roundCount);
          updateScreen = true;
          handled = true;
          break;
      }
      break;

    // a button was pressed
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {

        // close the form
        case btnID_close:
          FreeEditFixturesForm();
          FrmReturnToForm(0);
          handled = true;
          break;
        // add a match
        case btnID_addMatch:
          AddMatch();
          updateScreen = true;
          handled = true;
          break;
        // add a round
        case btnID_addRound:
          league = LeagueGetCurrent();
          LeagueAddRound(&league);
          GRound = league.roundCount;
          handled = true;
          updateScreen = true;
          break;
        // go to previous round
        case btnID_roundBack:
          GRound = max(GRound-1, 1);
          handled = true;
          updateScreen = true;
          break;
        // go to next round
        case btnID_roundFwd:
          league = LeagueGetCurrent();
          GRound = min(GRound+1, league.roundCount);
          handled = true;
          updateScreen = true;
          break;
        // date-control tapped
        case selectTrigID_date:
          year  = GDate.year + 1904;
          month = GDate.month;
          day   = GDate.day;
          SelectDay(selectDayByDay, &month, &day, &year, TEXT_MATCH_SELECT_MATCH_DAY);
          GDate.year  = year - 1904;
          GDate.month = month;
          GDate.day   = day;
          updateScreen = true;
          break;
      }
      break;

    // a menu item was selected
    case menuEvent:
      switch (event->data.menu.itemID) {
        // add round
        case menuitemID_fixtures_addround:
          league = LeagueGetCurrent();
          LeagueAddRound(&league);
          GRound = league.roundCount;
          updateScreen = true;
          break;
        // delete round
        case menuitemID_fixtures_deleteround:
          DeleteRound();
          updateScreen = true;
          break;
        // delete match
        case menuitemID_fixtures_deletematch:
          DeleteMatch();
          updateScreen = true;
          break;
      }
      break;

     // the screen was tapped
     // used when the user wants to jump to a round
      case penDownEvent:
        if(((event->screenX >= 54) && (event->screenX <= 90)) &&
           ((event->screenY >= 15) && (event->screenY <= 25))) {
          FrmPopupForm(formID_selectRound);
          handled = true;
        }
      break;

    default:
  }

  //  redraw the screen if necessary
  if(updateScreen) {
    UpdateEditFixturesForm();
    FrmDrawForm(FrmGetActiveForm());
    // underline the round label
    WinDrawGrayLine(56, 25, 84, 25); /*ToK: adjusted*/
  }

  return handled;
}


// ------------------------------------------------------------------
//
// Select round
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Draws a cell in the table. This is a callback function which is called
// when the table is redrawn.
// ------------------------------------------------------------------
void DrawRoundCell(void* tablePtr, Int16 row, Int16 column, RectanglePtr bounds) {
  Char   buf[5];
  Int16  round = (row*8) + column+1;
  League league = LeagueGetCurrent();

  if(round <= league.roundCount) {
    StrIToA(buf, round);
    UIDrawCharsCenter(buf, bounds->topLeft.x+(bounds->extent.x/2), bounds->topLeft.y);
  }
}


// ------------------------------------------------------------------
// Initializes "Select round" form.
// ------------------------------------------------------------------
void InitSelectRoundForm() {
  Int16    i;
  TablePtr tablePtr = (TablePtr)UIGetObjectPtr(tableID_rounds);

  UIInitCustomDrawTable(tableID_rounds, 8, 8, DrawRoundCell);
  for(i = 0; i < 8; i++) {
    TblSetRowHeight(tablePtr, i, 14);
  }
}


// ------------------------------------------------------------------
// Frees "Select round" form.
// ------------------------------------------------------------------
void FreeSelectRoundForm() {
}


// ------------------------------------------------------------------
// Eventhandler for "Select round" form.
// ------------------------------------------------------------------
Boolean SelectRoundFormEvtHandler(EventPtr event) {
  Boolean handled = false;
  Int16   row, col, round;
  UInt16  callingFormId;
  League  league;

  switch (event->eType) {

    // form is opened
    case frmOpenEvent:
      InitSelectRoundForm();
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;

   // form is closed
    case frmCloseEvent:
      FreeSelectRoundForm();
      break;

    // a button was pressed
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {

        // close the form
        case btnID_Cancel:
          FreeSelectRoundForm();
          FrmReturnToForm(0);
          handled = true;
          break;

      }
      break;

    // the screen was tapped
    // used when the user selects a round
    case penDownEvent:
      if(((event->screenX >= 2) && (event->screenX < 158)) &&
         ((event->screenY >= 18) && (event->screenY < 130))) {

        league = LeagueGetCurrent();
        row    = (event->screenY-18) / 14;
        col    = (event->screenX-2) / 19;
        round  = (row*8) + col+1;

        // make sure round is sane
        if(round <= league.roundCount) {
          // free resources and close form
          FreeSelectRoundForm();
          FrmReturnToForm(0);
          // update round and redraw the calling form
          callingFormId = FrmGetFormId(FrmGetFirstForm());
          if(callingFormId == formID_matches) {
            GState.Match.round = round;
            FrmUpdateForm(formID_matches, 0);
          }
          else if(callingFormId == formID_editFixtures) {
            GRound = round;
            FrmUpdateForm(formID_editFixtures, 0);
          }
        }
        handled = true;
      }
      break;

    default:
  }

  return handled;
}


// ------------------------------------------------------------------
//
// About
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Eventhandler for "About" form.
// ------------------------------------------------------------------
Boolean AboutFormEvtHandler(EventPtr event) {
  Boolean handled = false;

  switch (event->eType) {

    // form is opened
    case frmOpenEvent:
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;

    // a button was pressed
    case ctlSelectEvent:
      FrmReturnToForm(0);
      handled = true;
      break;

    default:
  }

  return handled;
}
