/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "League.h"
#include "TeamStats.h"
#include "Text.h"


// ------------------------------------------------------------------
// Globals declared elsewhere
// ------------------------------------------------------------------
extern DmOpenRef GCurrentDB;


// ------------------------------------------------------------------
// Returns the record index for the specified team.
// ------------------------------------------------------------------
UInt16 GetTeamRecordIndex(League* league, Int8 teamIndex) {
  return (1 + teamIndex);
}

// ------------------------------------------------------------------
// Returns the record index for the specified round.
// ------------------------------------------------------------------
UInt16 GetRoundRecordIndex(League* league, Int8 roundIndex) {
  return (1 + league->teamCount + roundIndex);
}

// ------------------------------------------------------------------
// Returns the record index for the specified match.
// ------------------------------------------------------------------
UInt16 GetMatchRecordIndex(League* league, Int16 matchIndex) {
  return (1 + league->teamCount + league->roundCount + matchIndex);
}


// ------------------------------------------------------------------
// Compares the dates of two matches.
// Returns 1 if m1 comes later than m2, -1 if m2 comes later, and 0 if they
// have the same date.
// ------------------------------------------------------------------
Int8 MatchDateCompare(Match* m1, Match* m2) {
  Int8 result = -1;

  if(m1->dateYear > m2->dateYear) {
    result = 1;
  }
  else if(m1->dateYear == m2->dateYear) {
    if(m1->dateMonth > m2->dateMonth) {
      result = 1;
    }
    else if(m1->dateMonth == m2->dateMonth) {
      if(m1->dateDay > m2->dateDay) {
        result = 1;
      }
      else if(m1->dateDay == m2->dateDay) {
        result = 0;
      }
    }
  }

  return result;
}


// ------------------------------------------------------------------
// Saves the league info in the first record of the db.
// ------------------------------------------------------------------
void LeagueSave(League* league) {
  MemHandle recHandle = DmGetRecord(league->dbRef, 0);
  void*     recPtr    = (void*)MemHandleLock(recHandle);
  Int8      dummy;

  DmWrite(recPtr, 0, &league->teamCount, 1);
  DmWrite(recPtr, 1, &league->roundCount, 1);
  DmWrite(recPtr, 2, &league->matchCount, 2);
  DmWrite(recPtr, 4, &league->playedMatchCount, 2);
  DmWrite(recPtr, 6, &league->winPoints, 1);
  DmWrite(recPtr, 7, &league->tableSortMethod, 1);
  // the following field is for future use
  dummy = -1;
  DmWrite(recPtr, 8, &dummy, 1);  // favourite team

  DmWrite(recPtr, 9,  &league->line[0], 1);
  DmWrite(recPtr, 10, &league->line[1], 1);
  DmWrite(recPtr, 11, &league->line[2], 1);
  DmWrite(recPtr, 12, &league->line[3], 1);
  DmWrite(recPtr, 13, &league->dbVersion, 1);
  DmWrite(recPtr, 14, &league->name, 17);
  DmWrite(recPtr, 31, &league->season, 2);

  MemHandleUnlock(recHandle);
  DmReleaseRecord(league->dbRef, 0, true);
}


// ------------------------------------------------------------------
// Saves the name of the specified team.
// ------------------------------------------------------------------
void LeagueSaveTeam(League* league, Int8 teamIndex, Char* teamName) {
  UInt16    recIndex  = GetTeamRecordIndex(league, teamIndex);
  MemHandle recHandle = DmGetRecord(league->dbRef, recIndex);
  Char*     recPtr    = (Char*)MemHandleLock(recHandle);

  DmWrite(recPtr, 0, teamName, 11);
  MemHandleUnlock(recHandle);
  DmReleaseRecord(league->dbRef, recIndex, true);
}


// ------------------------------------------------------------------
// Saves the handicap/penalty points of the specified team.
// ------------------------------------------------------------------
void LeagueSaveTeamPoints(League* league, Int8 teamIndex, Int8 points) {
  UInt16    recIndex  = GetTeamRecordIndex(league, teamIndex);
  MemHandle recHandle = DmGetRecord(league->dbRef, recIndex);
  void*     recPtr    = (void*)MemHandleLock(recHandle);

  DmWrite(recPtr, 11, &points, 1);
  MemHandleUnlock(recHandle);
  DmReleaseRecord(league->dbRef, recIndex, true);
}


// ------------------------------------------------------------------
// Adds a new round to the league. An empty round record is created
// and inserted into the database
// ------------------------------------------------------------------
void LeagueAddRound(League* league) {
  MemHandle recHandle;
  void*     recPtr;
  UInt16    index = GetRoundRecordIndex(league, league->roundCount); // index of new record
  Int8      value = 0;

  // create new record an get pointer to it
  recHandle = DmNewRecord(league->dbRef, &index, 1);
  recPtr    = (void*)MemHandleLock(recHandle);

  // write the empty record
  DmWrite(recPtr, 0, &value, 1);

  // unlock and realease
  MemHandleUnlock(recHandle);
  DmReleaseRecord(league->dbRef, index, true);

  // update and save league data structure
  league->roundCount++;
  LeagueSave(league);
}


// ------------------------------------------------------------------
// Returns a data structure for the specified open league database.
// ------------------------------------------------------------------
League LeagueGet(DmOpenRef dbRef) {
  League    league;
  // get a pointer to the first record
  MemHandle recHandle = DmQueryRecord(dbRef, 0);
  UInt8*    recPtr    = (UInt8*)MemHandleLock(recHandle);

  // copy data from the first record
  MemMove(&league.teamCount, recPtr, 1);
  MemMove(&league.roundCount, recPtr+1, 1);
  MemMove(&league.matchCount, recPtr+2, 2);
  MemMove(&league.playedMatchCount, recPtr+4, 2);
  MemMove(&league.winPoints, recPtr+6, 1);
  MemMove(&league.tableSortMethod, recPtr+7, 1);
  MemMove(&league.line[0], recPtr+9, 1);
  MemMove(&league.line[1], recPtr+10, 1);
  MemMove(&league.line[2], recPtr+11, 1);
  MemMove(&league.line[3], recPtr+12, 1);
  MemMove(&league.dbVersion, recPtr+13, 1);
  if (league.dbVersion >= 3) {
    MemMove(&league.name, recPtr+14, 17);
    MemMove(&league.season, recPtr+31, 2);
  }

  MemHandleUnlock(recHandle);
  league.dbRef = dbRef;

  // return the league data structure
  return league;
}


// ------------------------------------------------------------------
// Returns a data structure for the currently open league database.
// ------------------------------------------------------------------
League LeagueGetCurrent() {
  return LeagueGet(GCurrentDB);
}


// ------------------------------------------------------------------
// Returns a pointer to a team name. This pointer must later be unlocked
// using MemPtrUnlock().
// ------------------------------------------------------------------
Char* LeagueGetTeam(League* league, Int8 teamIndex) {
  MemHandle recHandle = DmQueryRecord(league->dbRef,
                                      GetTeamRecordIndex(league, teamIndex));
  return (Char*)MemHandleLock(recHandle);
}


// ------------------------------------------------------------------
// Returns the handicap/penalty points of the specified team.
// ------------------------------------------------------------------
Int8 LeagueGetTeamPoints(League* league, Int8 teamIndex) {
  Int8      points;
  MemHandle recHandle = DmQueryRecord(league->dbRef,
                                      GetTeamRecordIndex(league, teamIndex));
  UInt8*    recPtr    = (UInt8*)MemHandleLock(recHandle);

  MemMove(&points, recPtr+11, 1);
  MemHandleUnlock(recHandle);

  return points;
}


// ------------------------------------------------------------------
// Returns a match data structure.
// ------------------------------------------------------------------
Match LeagueGetMatch(League* league, Int16 matchIndex) {
  Match     match;
  MemHandle recHandle;
  UInt8*    recPtr;

  // get a pointer to the match record
  recHandle = DmQueryRecord(league->dbRef, GetMatchRecordIndex(league, matchIndex));
  recPtr    = (UInt8*)MemHandleLock(recHandle);

  // copy data to match data structure
  MemMove(&match.round, recPtr, 1);
  MemMove(&match.dateYear, recPtr+1, 2);
  MemMove(&match.dateMonth, recPtr+3, 1);
  MemMove(&match.dateDay, recPtr+4, 1);
  MemMove(&match.homeTeam, recPtr+5, 1);
  MemMove(&match.awayTeam, recPtr+6, 1);
  MemMove(&match.homeGoals, recPtr+7, 1);
  MemMove(&match.awayGoals, recPtr+8, 1);
  match.isPlayed = (match.homeGoals > -1);

  MemHandleUnlock(recHandle);

  return match;
}


// ------------------------------------------------------------------
// Finds all the matches in the specified round. The match indexes are placed
// in 'matches' and the function returns the number of matches found.
// ------------------------------------------------------------------
Int8 LeagueGetMatchesInRound(League* league, Int8 round, Int16* matches) {
  Int16     i;
  Int8      matchCount;
  MemHandle recHandle;
  void*     recPtr;

  // get a pointer to the round record
  recHandle = DmQueryRecord(league->dbRef, GetRoundRecordIndex(league, round-1));
  recPtr    = (void*)MemHandleLock(recHandle);

  // get number of matches in this round
  MemMove(&matchCount, recPtr, 1);
  // get the matches
  for(i = 0; i < matchCount; i++) {
    MemMove(&matches[i], recPtr+1+i*2, 2);
  }

  MemHandleUnlock(recHandle);

  return matchCount;
}


// ------------------------------------------------------------------
// Finds all the matches for the specified team. The parameter 'matches' indicates
// if home, away or all matches should be included. The match indexes are placed
// in 'matchesForTeam' and the function returns the number of matches found.
// ------------------------------------------------------------------
Int8 LeagueGetMatchesForTeam(League* league, Int8 teamIndex, Int8 matches, Int16* matchesForTeam) {
  Int16 i;
  Int8  matchCount = 0;
  Match match;

  // loop through all the matches and add matches for the specified team
  for(i = 0; i < league->matchCount; i++) {
    match = LeagueGetMatch(league, i);
    if(((match.homeTeam == teamIndex) && (matches == MATCHES_ALL || matches == MATCHES_HOME)) ||
       ((match.awayTeam == teamIndex) && (matches == MATCHES_ALL || matches == MATCHES_AWAY)))
    {
      matchesForTeam[matchCount++] = i;
    }
  }

  return matchCount;
}


// ------------------------------------------------------------------
// Checks if the specified date comes before the first match in the league.
// ------------------------------------------------------------------
Boolean IsBeforeFirstMatch(League* league, DateType* date) {
  Boolean isBeforeFirst = true;
  Boolean isBeforeFirstNotPlayed = true;
  Match   match, tempMatch;

  tempMatch.dateYear  = date->year + 1904;
  tempMatch.dateMonth = date->month;
  tempMatch.dateDay   = date->day;

  // check if it's before the first played match
  if (league->matchCount > 0) {
    match = LeagueGetMatch(league, 0);
    isBeforeFirst = (MatchDateCompare(&tempMatch, &match) == -1);
  }

  // check if it's before the first not yet played match
  if (league->playedMatchCount > league->matchCount) {
    match = LeagueGetMatch(league, league->playedMatchCount);
    isBeforeFirstNotPlayed = (MatchDateCompare(&tempMatch, &match) == -1);
  }

  return (isBeforeFirst && isBeforeFirstNotPlayed);
}


// ------------------------------------------------------------------
// Checks if the specified date comes after the last match in the league.
// ------------------------------------------------------------------
Boolean IsAfterLastMatch(League* league, DateType* date) {
  Boolean isAfterLast = true;
  Boolean isAfterLastPlayed = true;
  Match   match, tempMatch;

  tempMatch.dateYear  = date->year + 1904;
  tempMatch.dateMonth = date->month;
  tempMatch.dateDay   = date->day;

  // check if it's after the last not yet played match
  if (league->matchCount > 0) {
    match = LeagueGetMatch(league, league->matchCount-1);
    isAfterLast = (MatchDateCompare(&tempMatch, &match) == 1);
  }

  // check if it's after the last played match
  if (league->playedMatchCount > league->matchCount) {
    match = LeagueGetMatch(league, league->playedMatchCount-1);
    isAfterLastPlayed = (MatchDateCompare(&tempMatch, &match) == 1);
  }

  return (isAfterLast && isAfterLastPlayed);
}


// ------------------------------------------------------------------
// Finds all the matches for the specified date. The match indexes are placed
// in 'matchesForDate' and the function returns the number of matches found.
// ------------------------------------------------------------------
Int8 LeagueGetMatchesForDate(League* league, DateType* date, Int16* matchesForDate) {
  Int16 i;
  Int8  matchCount = 0;
  Int8  compareResult;
  Match match, tempMatch;

  // if the date is before the first match or after the last -> exit immediately
  if (IsBeforeFirstMatch(league, date) || IsAfterLastMatch(league, date)) {
    return 0;
  }

  tempMatch.dateYear  = date->year + 1904;
  tempMatch.dateMonth = date->month;
  tempMatch.dateDay   = date->day;

  // loop through all the played matches
  for (i = 0; i < league->playedMatchCount; i++) {
    match = LeagueGetMatch(league, i);
    compareResult = MatchDateCompare(&tempMatch, &match);
    // dates are equal
    if (compareResult == 0) {
      matchesForDate[matchCount++] = i;
    }
    // if we have passed the specified date we can exit the loop
    // since the matches are sorted by date
    if (compareResult == -1) {
      break;
    }
  }

  // loop through all the not yet played matches
  for (i = league->playedMatchCount; i < league->matchCount; i++) {
    match = LeagueGetMatch(league, i);
    compareResult = MatchDateCompare(&tempMatch, &match);
    // dates are equal
    if (compareResult == 0) {
      matchesForDate[matchCount++] = i;
    }
    // if we have passed the specified date we can exit the loop
    // since the matches are sorted by date
    if (compareResult == -1) {
      break;
    }
  }

  return matchCount;
}


// ------------------------------------------------------------------
// Returns the current round of the league.
// ------------------------------------------------------------------
Int16 LeagueGetCurrentRound(League* league) {
  Int16 result = 1;
  if(league->playedMatchCount > 0) {
    Match m = LeagueGetMatch(league, league->playedMatchCount-1);
    result  = m.round;
  }

  return result;
}


// ------------------------------------------------------------------
// Iterates the match list backwards starting at 'startIndex'. Returns the new index where
// 'match' should be stored (matches are sorted by date).
// ------------------------------------------------------------------
Int16 GetNewMatchIndex(League* league, Match* match, Int16 startIndex, Int16 stopIndex) {
  Int16 newIndex = startIndex+1;
  Match match2;

  do {
    newIndex--;
    if(stopIndex > newIndex) {
      break;
    }
    match2 = LeagueGetMatch(league, newIndex);
  } while((MatchDateCompare(match, &match2) < 0)/* && (stopIndex < newIndex)*/);

  return newIndex+1;
}


// ------------------------------------------------------------------
// Changes match indexes within round records. All indexes between 'lowestIndex'
// and 'highestIndex' are changed. The change (usually +1 or -1) is given
// in 'bumpFactor'
// ------------------------------------------------------------------
void LeagueBumpMatchIndex(League* league, Int16 lowestIndex, Int16 highestIndex, Int8 bumpFactor) {
  Int16     i, j;
  Int8      matchCount;
  UInt16    recIndex;
  MemHandle recHandle;
  void*     recPtr;
  Int16     matchIndex;

  // loop through all the rounds
  for(i = 0; i < league->roundCount; i++) {
    recIndex  = GetRoundRecordIndex(league, i);
    recHandle = DmGetRecord(league->dbRef, recIndex);
    recPtr    = (void*)MemHandleLock(recHandle);
    MemMove(&matchCount, recPtr, 1);

    // loop through all matches per round
    for(j = 0; j < matchCount; j++) {
      MemMove(&matchIndex, recPtr+1+j*2, 2);
      // bump the index if it's within the boundries
      if((matchIndex >= lowestIndex) && (matchIndex <= highestIndex)) {
        matchIndex += bumpFactor;
        DmWrite(recPtr, 1+j*2, &matchIndex, 2);
      }
    }

    MemHandleUnlock(recHandle);
    DmReleaseRecord(league->dbRef, recIndex, true);
  }

}


// ------------------------------------------------------------------
// Saves a new result for the specified match.
// Returns the new matchIndex.
// ------------------------------------------------------------------
Int16 LeagueUpdateMatchResult(League* league, Int16 matchIndex, Int8 homeGoals, Int8 awayGoals) {
  Int8  roundPos;
  Int16 newMatchIndex = matchIndex;
  // get ref to the match
  Match match = LeagueGetMatch(league, matchIndex);

  // only make changes if date is different
  if((homeGoals != match.homeGoals) ||
     (awayGoals != match.awayGoals)) {

    // Changing the result can mean that the match will be stored at
    // another position in the db (since played and not yet played
    // matches are stored separately in the db).
    // The easiest way to handle this is to first delete the match
    // and then add it again with the new date.

    // delete the match
    roundPos = LeagueDeleteMatch(league, match.round, matchIndex);
    // change the result
    match.homeGoals = homeGoals;
    match.awayGoals = awayGoals;
    match.isPlayed  = (homeGoals > -1);
    // add the match again
    newMatchIndex = LeagueAddMatch(league, &match, roundPos);
  }

  return newMatchIndex;
}


// ------------------------------------------------------------------
// Return the specified match as a string.
// ------------------------------------------------------------------
void LeagueGetMatchString(League* league, Int16 matchIndex, Char* matchString) {
  Match match    = LeagueGetMatch(league, matchIndex);
  Char* homeTeam = LeagueGetTeam(league, match.homeTeam);
  Char* awayTeam = LeagueGetTeam(league, match.awayTeam);

  if(match.isPlayed) {
    StrPrintF(matchString, TEXT_LEAGUE_MATCH_PLAYED, match.dateDay, match.dateMonth,
        homeTeam, awayTeam, match.homeGoals, match.awayGoals);
  }
  else {
    StrPrintF(matchString, TEXT_LEAGUE_MATCH_FUTURE, match.dateDay, match.dateMonth, homeTeam, awayTeam);
  }

  MemPtrUnlock(homeTeam);
  MemPtrUnlock(awayTeam);
}


// ------------------------------------------------------------------
// Return the specified match as a string without date.
// ------------------------------------------------------------------
void LeagueGetMatchStringNoDate(League* league, Int16 matchIndex, Char* matchString) {
  Match match    = LeagueGetMatch(league, matchIndex);
  Char* homeTeam = LeagueGetTeam(league, match.homeTeam);
  Char* awayTeam = LeagueGetTeam(league, match.awayTeam);

  if(match.isPlayed) {
    StrPrintF(matchString, TEXT_LEAGUE_MATCH_NO_DATE_PLAYED,
        homeTeam, awayTeam, match.homeGoals, match.awayGoals);
  }
  else {
    StrPrintF(matchString, TEXT_LEAGUE_MATCH_NO_DATE_FUTURE, homeTeam, awayTeam);
  }

  MemPtrUnlock(homeTeam);
  MemPtrUnlock(awayTeam);
}


// ------------------------------------------------------------------
// Inserts a new match record at the given index.
// ------------------------------------------------------------------
void InsertMatch(League* league, Match* match, Int16 matchIndex) {
  UInt16    recIndex;
  MemHandle recHandle;
  void*     recPtr;

  // increase match count and played match count
  league->matchCount++;
  if(match->isPlayed) {
    league->playedMatchCount++;
  }
  LeagueSave(league);

  // create new record
  recIndex  = GetMatchRecordIndex(league, matchIndex);
  recHandle = DmNewRecord(league->dbRef, &recIndex, 9);
  recPtr    = (void*)MemHandleLock(recHandle);

  // store the data
  DmWrite(recPtr, 0, &match->round, 1);
  DmWrite(recPtr, 1, &match->dateYear, 2);
  DmWrite(recPtr, 3, &match->dateMonth, 1);
  DmWrite(recPtr, 4, &match->dateDay, 1);
  DmWrite(recPtr, 5, &match->homeTeam, 1);
  DmWrite(recPtr, 6, &match->awayTeam, 1);
  DmWrite(recPtr, 7, &match->homeGoals, 1);
  DmWrite(recPtr, 8, &match->awayGoals, 1);
  MemHandleUnlock(recHandle);

  DmReleaseRecord(league->dbRef, recIndex, true);
}


// ------------------------------------------------------------------
// Adds a new match to the given round record.
// RoundPos is the position the new match should have in the round. To
// add the new match ordered by date specify -1 as RoundPos.
// ------------------------------------------------------------------
void RoundAddMatch(League* league, Int8 round, Int16 matchIndex, Int8 roundPos) {
  Int16     i;
  Int16     tempMatchIndex;
  Int8      matchCount;
  UInt16    recIndex;
  MemHandle recHandle;
  void*     recPtr;
  Match     newMatch;
  Match     tempMatch;

  // get a pointer to the round record
  recIndex  = GetRoundRecordIndex(league, round-1);
  recHandle = DmGetRecord(league->dbRef, recIndex);
  recPtr    = (void*)MemHandleLock(recHandle);
  // get number of matches in this round
  MemMove(&matchCount, recPtr, 1);

  newMatch = LeagueGetMatch(league, matchIndex);
  i = 0;
  // loop logic:
  // Matches within the round record are sorted by date.
  // We loop through the matches and when we find the position
  // for the new match (either by date or by RoundPos, if specified)
  // we insert it there and then make a recursive call to
  // insert the match we wrote over.
  while(i < matchCount) {
    MemMove(&tempMatchIndex, recPtr+1+i*2, 2);
    tempMatch = LeagueGetMatch(league, tempMatchIndex);

    if(((roundPos > -1) && (roundPos == i)) ||
       (MatchDateCompare(&newMatch, &tempMatch) == -1)) {
      // insert the match
      DmWrite(recPtr, 1+i*2, &matchIndex, 2);
      // unlock and release
      MemHandleUnlock(recHandle);
      DmReleaseRecord(league->dbRef, recIndex, true);
      // make recursive call with the previous match
      RoundAddMatch(league, round, tempMatchIndex, i+1);
      break;
    }

    i++;
  }

  // if we have looped through all the matches the new
  // match should be inserted last.
  if(i == matchCount) {
    MemHandleUnlock(recHandle);
    DmReleaseRecord(league->dbRef, recIndex, true);

    // increase the matchcount for this round and resize the record
    matchCount++;
    DmResizeRecord(league->dbRef, recIndex, 1+matchCount*2);

    // get new pointer the round record
    recHandle = DmGetRecord(league->dbRef, recIndex);
    recPtr    = (void*)MemHandleLock(recHandle);
    // store the new matchcount...
    DmWrite(recPtr, 0, &matchCount, 1);
    // ... and the new match
    DmWrite(recPtr, 1+(matchCount-1)*2, &matchIndex, 2);
    // unlock and release
    MemHandleUnlock(recHandle);
    DmReleaseRecord(league->dbRef, recIndex, true);
  }

}


// ------------------------------------------------------------------
// Adds a new match to the league.
// Returns the index of the match.
// ------------------------------------------------------------------
Int16 LeagueAddMatch(League* league, Match* match, Int8 roundPos) {
  Int16 matchIndex;

  // Get index for the new match. Matches are in two sections
  // in the db: one with played matches and one with not yet played.
  // Both sections are sorted by date.
  if(match->isPlayed) {
    matchIndex = GetNewMatchIndex(league, match, league->playedMatchCount-1, 0);
  }
  else {
    matchIndex = GetNewMatchIndex(league, match, league->matchCount-1, league->playedMatchCount);
  }
  // add the new match record
  InsertMatch(league, match, matchIndex);
  // since we inserted a match some match indexes within the round records
  // need to be increased by one
  LeagueBumpMatchIndex(league, matchIndex, league->matchCount-1, 1);
  // add the match to the round record
  RoundAddMatch(league, match->round, matchIndex, roundPos);

  return matchIndex;
}


// ------------------------------------------------------------------
// Removes the match record from the DB.
// ------------------------------------------------------------------
void DeleteMatchRecord(League* league, Int16 matchIndex) {
  UInt16 recIndex;

  // decrease match count and played match count
  league->matchCount--;
  if(matchIndex < league->playedMatchCount) {
    league->playedMatchCount--;
  }
  LeagueSave(league);

  // remove the record
  recIndex = GetMatchRecordIndex(league, matchIndex);
  DmRemoveRecord(league->dbRef, recIndex);
}


// ------------------------------------------------------------------
// Deletes a match from a round record.
// Returns the position the match had in the round.
// ------------------------------------------------------------------
Int8 RoundDeleteMatch(League* league, Int8 round, Int16 matchIndex) {
  Int8      i;
  Int8      roundPos;
  Int16     tempMatchIndex;
  Int8      matchCount;
  UInt16    recIndex;
  MemHandle recHandle;
  void*     recPtr;

  // get a pointer to the round record
  recIndex  = GetRoundRecordIndex(league, round-1);
  recHandle = DmGetRecord(league->dbRef, recIndex);
  recPtr    = (void*)MemHandleLock(recHandle);
  // get number of matches in this round
  MemMove(&matchCount, recPtr, 1);

  // loop until we find the match...
  i = 0;
  while(i < matchCount) {
    MemMove(&tempMatchIndex, recPtr+1+i*2, 2);
    if(tempMatchIndex == matchIndex) {
      break;
    }
    i++;
  }

  roundPos = i;

  // ...and then move all the following matches
  // one step "down"
  while(i < matchCount-1) {
    MemMove(&tempMatchIndex, recPtr+1+(i+1)*2, 2);
    DmWrite(recPtr, 1+i*2, &tempMatchIndex, 2);
    i++;
  }

  // decrease match count
  matchCount--;
  DmWrite(recPtr, 0, &matchCount, 1);
  // unlock and release
  MemHandleUnlock(recHandle);
  DmReleaseRecord(league->dbRef, recIndex, true);
  // the record is now smaller since we removed one match
  DmResizeRecord(league->dbRef, recIndex, 1+matchCount*2);

  return roundPos;
}


// ------------------------------------------------------------------
// Deletes the specified match.
// Returns the position the match had in the round.
// ------------------------------------------------------------------
Int8 LeagueDeleteMatch(League* league, Int8 round, Int16 matchIndex) {
  Int8 roundPos;

  // remove the match record
  DeleteMatchRecord(league, matchIndex);
  // remove the match from roundrecord
  roundPos = RoundDeleteMatch(league, round, matchIndex);
  // bump match indexes
  LeagueBumpMatchIndex(league, matchIndex+1, league->matchCount, -1);

  return roundPos;
}


// ------------------------------------------------------------------
// Deletes the specified round. This function assumes that
// the round is the last one and that it doesn't contain
// any matches.
// ------------------------------------------------------------------
void LeagueDeleteRound(League* league, Int8 round) {
  UInt16 recIndex;

  // decrease round count
  league->roundCount--;
  LeagueSave(league);
  // remove the record
  recIndex = GetRoundRecordIndex(league, round-1);
  DmRemoveRecord(league->dbRef, recIndex);
}


// ------------------------------------------------------------------
// Changes the date of the specified match.
// Returns the new index of the match.
// ------------------------------------------------------------------
Int16 LeagueChangeMatchDate(League* league, Int16 matchIndex, Int16 year, Int16 month, Int16 day) {
  Int16 newMatchIndex = matchIndex;
  Match match = LeagueGetMatch(league, matchIndex);

  // only store changes if date has changed
  if((year  != match.dateYear) ||
     (month != match.dateMonth) ||
     (day   != match.dateDay)) {

    // Changing the date can mean that the match will be stored at
    // another position in the db (since matches are sorted by date).
    // The easiest way to handle this is to first delete the match
    // and then add it again with the new date.

    // delete the match
    LeagueDeleteMatch(league, match.round, matchIndex);
    // change the date
    match.dateYear  = year;
    match.dateMonth = month;
    match.dateDay   = day;
    // add the match again with the new date
    newMatchIndex = LeagueAddMatch(league, &match, -1);
  }

  return newMatchIndex;
}


// ------------------------------------------------------------------
// Sorts the teams of the league alpabetically. The indexes of the teams
// are placed in the array 'sortedIndex'.
// ------------------------------------------------------------------
void LeagueGetSortedTeamIndexes(League* league, Int16* sortedIndex) {
  Int16 i, j;
  Int16 pos, compareRes;
  Char* team1;
  Char* team2;

  // init
  for(i = 0; i < league->teamCount; i++) {
    sortedIndex[i] = 0;
  }

  // loop through team names and sort them
  for(i = 0; i < league->teamCount; i++) {
    pos = 0;
    team1 = LeagueGetTeam(league, i);
    for(j = 0; j < league->teamCount; j++) {
      team2 = LeagueGetTeam(league, j);
      compareRes = StrCompare(team1, team2);
      //compareRes = TxtCompare(team1, StrLen(team2), NULL, team2, StrLen(team2), NULL);
      if((compareRes > 0) || (compareRes == 0 && i > j)) {
        pos++;
      }
      MemPtrUnlock(team2);
    }
    MemPtrUnlock(team1);
    sortedIndex[pos] = i;
  }

}


// ------------------------------------------------------------------
// Upgrades an old database (version 2 or below) to version 3. The name of the
// league and the season are added to the first record.
// ------------------------------------------------------------------
void LeagueUpgradeDatabase(DmOpenRef dbRef) {
  League  league = LeagueGet(dbRef);
  Char    dbName[32];
  LocalID localId;
  UInt16  cardNo;

  // we use the database name as leaguename
  DmOpenDatabaseInfo(dbRef, &localId, NULL, NULL, &cardNo, NULL);
  DmDatabaseInfo(cardNo, localId, dbName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  dbName[16] = '\0';
  StrCopy(league.name, dbName);

  // if matches are played we check the match dates to determine the season
  if (league.matchCount > 0) {
    UInt16 startYear, endYear;
    Match  match;

    match     = LeagueGetMatch(&league, 0);
    startYear = match.dateYear;
    match     = LeagueGetMatch(&league, league.matchCount-1);
    endYear   = match.dateYear;
    // if the league is split over two years we save the season as a negative number
    league.season = (startYear == endYear) ? startYear : -startYear;
  }
  // if no matches are played we use the current year
  else {
    DateType today;
    DateSecondsToDate(TimGetSeconds(), &today);
    league.season = today.year + 1904;
  }

  // upgrade the db version
  league.dbVersion = 3;

  // resize the first record and save the league data
  DmResizeRecord(dbRef, 0, 33);
  LeagueSave(&league);

}
