/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "LeagueDialogs.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Text.h"

#define createLeagueErr 1


// ------------------------------------------------------------------
// Globals declared elsewhere
// ------------------------------------------------------------------
extern AppState    GState;
extern DmOpenRef   GCurrentDB;
extern LeagueInfo* GLeagueInfo;
extern Char**      GLeagueNameLabels;
// ------------------------------------------------------------------
// Globals used by "Edit league"
// ------------------------------------------------------------------
MemHandle GLeagueNameHandle;
Char*     GLineNames[MAX_TEAMS + 1];
// ------------------------------------------------------------------
// Globals used by "Todays matches"
// ------------------------------------------------------------------
Char*    GTodayStrings[MAX_TODAYS_MATCHES];
Int16    GStringCount;
DateType GDate;
Char     GWeekDays[7][4] = { TEXT_TODAYS_MATCHES_SUN,
                             TEXT_TODAYS_MATCHES_MON,
                             TEXT_TODAYS_MATCHES_TUE,
                             TEXT_TODAYS_MATCHES_WED,
                             TEXT_TODAYS_MATCHES_THU,
                             TEXT_TODAYS_MATCHES_FRI,
                             TEXT_TODAYS_MATCHES_SAT };



// ------------------------------------------------------------------
// Callback function used when beaming database.
// ------------------------------------------------------------------
Err WriteDBData(const void* dataP, UInt32* sizeP, void* userDataP) {
  Err err;

  *sizeP = ExgSend((ExgSocketPtr)userDataP, (void*)dataP, *sizeP, &err);

  return err;
}


// ------------------------------------------------------------------
// Beams the current the database.
// ------------------------------------------------------------------
void BeamCurrentDatabase() {
  ExgSocketType exgSocket;
  Err           err;
  Char          filename[64];
  Char          description[64];

  MemSet(&exgSocket, sizeof(exgSocket), 0);
  DmDatabaseInfo(GState.currentLeagueInfo->cardNo, GState.currentLeagueInfo->localID, filename, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  StrCopy(description, filename);
  StrCat(filename, ".pdb");
  exgSocket.description = description;
  exgSocket.name        = filename;

  err = ExgPut(&exgSocket);
  if(!err) {
    err = ExgDBWrite(WriteDBData, &exgSocket, NULL,
            GState.currentLeagueInfo->localID,
            GState.currentLeagueInfo->cardNo);
    err = ExgDisconnect(&exgSocket, err);
  }

}


// ------------------------------------------------------------------
// Opens a database to use as the current league database.
// ------------------------------------------------------------------
void OpenDatabase(UInt16 cardNo, LocalID dbID) {
  GCurrentDB = DmOpenDatabase(cardNo, dbID, dmModeReadWrite);
}


// ------------------------------------------------------------------
// Closes the current database.
// ------------------------------------------------------------------
void CloseCurrentDatabase() {
  if(GCurrentDB) {
    DmCloseDatabase(GCurrentDB);
    GCurrentDB = NULL;
  }
}


// ------------------------------------------------------------------
// Deletes the currently open database.
// ------------------------------------------------------------------
void DeleteCurrentDatabase() {
  CloseCurrentDatabase();
  DmDeleteDatabase(GState.currentLeagueInfo->cardNo, GState.currentLeagueInfo->localID);
  // this call makes sure that the deleted database is removed from the global variables
  // containing db info and league name.
  InitDatabases();
}



// ------------------------------------------------------------------
//
// Create league
//
// ------------------------------------------------------------------


// ------------------------------------------------------------------
// Creates the name of the database by merging the name
// of the league and the season. The name is returned in nameBuf.
// ------------------------------------------------------------------
void CreateLeagueDbName(League *league, Char *nameBuf) {
  Char yearBuf[5];

  if (league->season < 0) {
    Int16 season = abs(league->season) % 100;
    StrPrintF(yearBuf, "%2d%2d", season, (season+1)%100);
    if (season < 9) yearBuf[0] = '0';
    if ((season+1)%100 < 9) yearBuf[2] = '0';
  }
  else {
    Int16 season = league->season % 100;
    StrPrintF(yearBuf, "%2d", season);
    if (season < 9) yearBuf[0] = '0';
  }

  StrPrintF(nameBuf, "%s%s", league->name, yearBuf);
}


// ------------------------------------------------------------------
// Creates a new database with the specified name and league information.
// Returns any errors that occured when the db was created.
// ------------------------------------------------------------------
Err CreateDatabase(League *league) {
  Int8    i;
  Err     err;
  LocalID dbId;
  UInt16  index = 0;
  UInt16  dbAttrs;
  Char    dbName[32];

  // create the league db name
  CreateLeagueDbName(league, dbName);

  // try to create the database
  err = DmCreateDatabase(0, dbName, 'mbTS', 'Data', false);
  if(err) {
    return err;
  }

  // find the newly created database
  dbId = DmFindDatabase(0, dbName);
  // set the backup flag so the db is copied to the desktop PC during sync
  dbAttrs = dmHdrAttrBackup;
  DmSetDatabaseInfo(0, dbId, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

  // open the db
  league->dbRef = DmOpenDatabase(0, dbId, dmModeReadWrite);
  // set some default values
  league->dbVersion        = 3;
  league->matchCount       = 0;
  league->playedMatchCount = 0;
  league->roundCount       = 0;
  league->tableSortMethod  = 0;
  league->line[0] = league->line[1] = league->line[2] = league->line[3] = -1;

  // create the first record...
  DmNewRecord(league->dbRef, &index, 33);
  DmReleaseRecord(league->dbRef, index, true);
  // ... so we can store the league info there.
  LeagueSave(league);

  // create records for the teams in the league and store a default name for each team
  index++;
  for(i = 0; i < league->teamCount; i++) {
    Char buf[10];
    StrPrintF(buf, TEXT_EDIT_LEAGUE_TEAM, i+1);
    DmNewRecord(league->dbRef, &index, 12);
    DmReleaseRecord(league->dbRef, index, true);
    LeagueSaveTeam(league, i, buf);
    LeagueSaveTeamPoints(league, i, 0);
    index++;
  }

  // add the first (empty) round
  LeagueAddRound(league);

  DmCloseDatabase(league->dbRef);

  // add info about the new db to global variables
  AddLeagueInfo(dbId, 0);
  PrepareLeagueNameLabels();

  // return no error
  return 0;
}


// ------------------------------------------------------------------
// Tries to create a new league from information in the "Create league" form.
// Returns any error that ocurred when the db was created or 'createLeagueErr'
// if the form wasn't correctly filled in.
// ------------------------------------------------------------------
Err SubmitCreateLeagueForm() {
  MemHandle textHandle;
  Char*     namePtr;
  League    league;
  Err       err = 0;
  UInt16    pointGroupIndex;
  FormPtr   form = FrmGetActiveForm();

  // get winpoints, teamcount, and season from form
  pointGroupIndex  = FrmGetControlGroupSelection(form, groupID_winPoints);
  league.winPoints = (FrmGetObjectId(form, pointGroupIndex) == btnID_3point) ? 3 : 2;
  league.teamCount = UIGetFieldIntValue(fieldID_teamCount);
  league.season    = StrAToI(FrmGetLabel(form, labelID_year));
  // if season is split over two years we save it as a negative number
  if (FrmGetControlValue(form, FrmGetObjectIndex(form, cbID_split)) == 1) {
    league.season = -league.season;
  }

  // get handle for name field
  textHandle = FldGetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_leaguename));

  if(textHandle != NULL) {
    namePtr = MemHandleLock(textHandle);
    StrCopy(league.name, namePtr);
    // if the entered info is ok -> go ahead and create the db
    if((league.teamCount > 1) && (league.teamCount <= MAX_TEAMS) && (StrLen(league.name) > 0)) {
      err = CreateDatabase(&league);
    }
    else {
      err = createLeagueErr;
    }

    MemHandleUnlock(textHandle);
  }
  else {
    err = createLeagueErr;
  }

  return err;
}


// ------------------------------------------------------------------
// Changes the season label. If yearDelta is 0 (zero) the label is set
// to the current year. Otherwise the value of the label is changed with
// yearDelta.
// ------------------------------------------------------------------
void UpdateSeasonLabel(Int8 yearDelta) {
  Char    buf[5];
  Int16   year;
  FormPtr form = FrmGetActiveForm();

  if (yearDelta == 0) {
    DateType today;
    DateSecondsToDate(TimGetSeconds(), &today);
    year = today.year + 1904;
  }
  else {
    year = StrAToI(FrmGetLabel(form, labelID_year)) + yearDelta;
  }

  StrIToA(buf, year);
  FrmHideObject(form, FrmGetObjectIndex(form, labelID_year));
  FrmCopyLabel(form, labelID_year, buf);
  FrmShowObject(form, FrmGetObjectIndex(form, labelID_year));
}


// ------------------------------------------------------------------
// Eventhandler for "Create league" form.
// ------------------------------------------------------------------
Boolean CreateLeagueFormEvtHandler(EventPtr event) {
  Boolean handled = false;
  Err     err;

  switch (event->eType) {

   // form is opened
    case frmOpenEvent:
      UpdateSeasonLabel(0);
      FrmSetControlGroupSelection(FrmGetActiveForm(), groupID_winPoints, btnID_3point);
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;

   // a button was pressed
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {

        // arrow buttons for the season
        case btnID_yearUp:
          UpdateSeasonLabel(1);
          FrmDrawForm(FrmGetActiveForm());
          handled = true;
          break;

        case btnID_yearDown:
          UpdateSeasonLabel(-1);
          FrmDrawForm(FrmGetActiveForm());
          handled = true;
          break;

        // OK -> try and create the new league
        case btnID_OK:
          err = SubmitCreateLeagueForm();
          switch(err) {
            case 0: // no error
              FrmReturnToForm(0);
              ShowLeague(GState.currentLeagueNameIndex, GState.currentSeasonIndex, MODE_TABLE, true, true);
              break;
            case createLeagueErr:
              FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_NAME_NO_OF_TEAMS, NULL, NULL);
              break;
            case dmErrInvalidDatabaseName:
            case dmErrAlreadyExists:
              FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_INVALID_NAME, NULL, NULL);
              break;
            default: // all other errors (not handled at the moment)
              FrmReturnToForm(0);
          }
          handled = true;
          break;

        // Cancel -> just return to previous form
        case btnID_Cancel:
          FrmReturnToForm(0);
          handled = true;
          break;
      }
      break;

    default:
  }

  return handled;
}


// ------------------------------------------------------------------
//
// Edit league
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Edits the currently open database. 'isLeagueNameChanged' specifies
// if the league name or season has changed.
// Returns any errors that occured when the db name was altered.
// ------------------------------------------------------------------
Err EditCurrentDatabase(League *league, Boolean isLeagueNameChanged) {
  Err  err = 0;
  Char oldDbName[32];
  Char newDbName[32];
  LocalID dbId   = GState.currentLeagueInfo->localID;
  UInt16  cardNo = GState.currentLeagueInfo->cardNo;

  // get the old db name
  DmDatabaseInfo(cardNo, dbId,
                 oldDbName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  // create a new db name
  CreateLeagueDbName(league, newDbName);

  // change the db name if necessary
  if (StrCompare(oldDbName, newDbName) != 0) {
    err = DmSetDatabaseInfo(cardNo, dbId,
                            newDbName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  }

  // if no error -> save the league data
  if(!err) {
    LeagueSave(league);

    if (isLeagueNameChanged) {
      // if the leaguename changed we need to re-initialize the league structures
      CloseCurrentDatabase();
      InitDatabases();
      OpenDatabase(cardNo, dbId);
    }
  }

  return err;
}


// ------------------------------------------------------------------
// Saves the new league info from the "Edit league" form.
// ------------------------------------------------------------------
Err SubmitEditLeagueForm() {
  Int16   i;
  Err     err = 0;
  Char*   namePtr;
  ListPtr lineList;
  ListPtr tableSortList;
  UInt16  pointGroupIndex;
  Int16   newSeason;
  FormPtr form = FrmGetActiveForm();
  League  league = LeagueGetCurrent();

  // get the season from form
  newSeason = StrAToI(FrmGetLabel(form, labelID_year));
  // if season is split over two years we save it as a negative number
  if (FrmGetControlValue(form, FrmGetObjectIndex(form, cbID_split)) == 1) {
    newSeason = -newSeason;
  }

  // get winpoints from form
  pointGroupIndex  = FrmGetControlGroupSelection(FrmGetActiveForm(), groupID_winPoints);
  league.winPoints = (FrmGetObjectId(FrmGetActiveForm(), pointGroupIndex) == btnID_3point) ? 3 : 2;

  // get table sort method from form
  tableSortList = (ListPtr)UIGetObjectPtr(popListID_tableSort);
  league.tableSortMethod = LstGetSelection(tableSortList);

  // get lines from form
  for (i = 0; i < 4; i++) {
    lineList = (ListPtr)UIGetObjectPtr(popListID_line0 + i);
    league.line[i] = LstGetSelection(lineList) - 1;
  }

  // check that a name was entered and save the league
  namePtr = MemHandleLock(GLeagueNameHandle);
  if(StrLen(namePtr) > 0) {
    // if league name or season was changed we need to change the db name
    Boolean isLeagueNameChanged = (StrCompare(namePtr, league.name) || league.season != newSeason);
    league.season = newSeason;
    StrCopy(league.name, namePtr);
    // try to save the changes
    err = EditCurrentDatabase(&league, isLeagueNameChanged);
  }
  else {
    err = createLeagueErr;
  }

  MemHandleUnlock(GLeagueNameHandle);

  return err;
}


// ------------------------------------------------------------------
// Initializes "Edit league" form.
// ------------------------------------------------------------------
void InitEditLeagueForm() {
  Int16   i;
  Char    buf[5];
  UInt16  btnID;
  ListPtr lineList;
  Char*   namePtr;
  FormPtr form = FrmGetActiveForm();
  League  league = LeagueGetCurrent();

  // create and set text handle for the league name field
  GLeagueNameHandle = MemHandleNew(LEAGUENAME_SIZE);
  namePtr = (Char*)MemHandleLock(GLeagueNameHandle);
  StrCopy(namePtr, league.name);
  MemHandleUnlock(GLeagueNameHandle);
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_leaguename), GLeagueNameHandle);

  // season label and "split season"-checkbox
  StrIToA(buf, abs(league.season));
  FrmCopyLabel(form, labelID_year, buf);
  FrmSetControlValue(form, FrmGetObjectIndex(form, cbID_split), (league.season < 0));

  // select the correct radio button
  btnID = (league.winPoints == 3) ? btnID_3point : btnID_2point;
  FrmSetControlGroupSelection(form, groupID_winPoints, btnID);

  // set table sort method in drop-down
  UISelectDropDown(popListID_tableSort, popTrigID_tableSort, league.tableSortMethod);

  // allocate memory and initialize the data for the line drop-downs
  GLineNames[0] = (Char*)MemPtrNew(3);
  StrCopy(GLineNames[0], "-");
  for(i = 0; i < MAX_TEAMS; i++) {
    if (i < league.teamCount) {
      GLineNames[i+1] = (Char*)MemPtrNew(3);
      StrIToA(GLineNames[(i+1)], i+1);
    }
    else {
      GLineNames[i+1] = NULL;
    }
  }
  // give the pointer array to the list-controls
  for (i = 0; i < 4; i++) {
    lineList = (ListPtr)UIGetObjectPtr(popListID_line0 + i);
    LstSetListChoices(lineList, GLineNames, league.teamCount + 1);
    UISelectDropDown(popListID_line0 + i, popTrigID_line0 + i, league.line[i] + 1);
  }

}


// ------------------------------------------------------------------
// Frees resources allocated for this form.
// ------------------------------------------------------------------
void FreeEditLeagueForm() {
  Int16  i;

  // free memory for line drop-downs
  for (i = 0; i < MAX_TEAMS+1; i++) {
    if (GLineNames[i]) {
      MemPtrFree(GLineNames[i]);
    }
  }
  // free the handle used by league name field
  FldSetTextHandle((FieldPtr)UIGetObjectPtr(fieldID_leaguename), NULL);
  MemHandleFree(GLeagueNameHandle);
}


// ------------------------------------------------------------------
// Eventhandler for "Edit league" form.
// ------------------------------------------------------------------
Boolean EditLeagueFormEvtHandler(EventPtr event) {
  Boolean handled = false;
  Err     err;

  switch (event->eType) {

   // form is opened
    case frmOpenEvent:
      InitEditLeagueForm();
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;

   // form is closed
    case frmCloseEvent:
      FreeEditLeagueForm();
      break;

   // a button was pressed
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {

        // arrow buttons for the season
        case btnID_yearUp:
          UpdateSeasonLabel(1);
          FrmDrawForm(FrmGetActiveForm());
          handled = true;
          break;

        case btnID_yearDown:
          UpdateSeasonLabel(-1);
          FrmDrawForm(FrmGetActiveForm());
          handled = true;
          break;

        // OK -> try and edit the league data
        case btnID_OK:
          err = SubmitEditLeagueForm();
          switch(err) {
            case 0: // no error
              FreeEditLeagueForm();
              FrmReturnToForm(0);
              ShowLeague(GState.currentLeagueNameIndex, GState.currentSeasonIndex, GState.mode, true, true);
              break;
            case createLeagueErr:
              FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_MUST_HAVE_NAME, NULL, NULL);
              break;
            case dmErrInvalidDatabaseName:
            case dmErrAlreadyExists:
              FrmCustomAlert(alertID_errGeneral, TEXT_EDIT_LEAGUE_INVALID_NAME, NULL, NULL);
              break;
            default: // all other errors (not handled at the moment)
              FreeEditLeagueForm();
              FrmReturnToForm(0);
          }
          handled = true;
          break;

        // Cancel -> just return to previous form
        case btnID_Cancel:
          FreeEditLeagueForm();
          FrmReturnToForm(0);
          handled = true;
          break;
      }
      break;

    default:
  }

  return handled;
}


// ------------------------------------------------------------------
//
// League order
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Changes the positions of two leagues in the league list
// ------------------------------------------------------------------
void SwapLeagues(Int16 league1, Int16 league2) {

  if ((league2 >= 0) && (league2 < GState.leagueCount)) {
    // update league name list
    Char *tempPtr = GLeagueNameLabels[league2];
    GLeagueNameLabels[league2] = GLeagueNameLabels[league1];
    GLeagueNameLabels[league1] = tempPtr;
  }
}


// ------------------------------------------------------------------
// Moves the selected league up or down, delta is either -1 or 1.
// ------------------------------------------------------------------
void MoveLeague(Int16 delta) {

  Int16 selectedIndex = LstGetSelection((ListPtr)UIGetObjectPtr(listID_leagues));

  // make sure a league is selectd and that we're not moving up from
  // the first item or down from the last.
  if ((selectedIndex > -1) &&
      (selectedIndex + delta >= 0) &&
      (selectedIndex + delta < GState.leagueCount)) {

    ListPtr leagueList = (ListPtr)UIGetObjectPtr(listID_leagues);

    SwapLeagues(selectedIndex, selectedIndex + delta);
    // re-draw the list
    LstSetListChoices(leagueList, GLeagueNameLabels, GState.leagueNameCount);
    LstSetSelection(leagueList, selectedIndex + delta);
    LstDrawList(leagueList);
  }
}


// ------------------------------------------------------------------
// Initializes "League order" form.
// ------------------------------------------------------------------
void InitLeagueOrderForm() {
  ListPtr leagueList = (ListPtr)UIGetObjectPtr(listID_leagues);
  LstSetListChoices(leagueList, GLeagueNameLabels, GState.leagueNameCount);
}


// ------------------------------------------------------------------
// Frees resources allocated for this form.
// ------------------------------------------------------------------
void FreeLeagueOrderForm() {
}


// ------------------------------------------------------------------
// Eventhandler for "League order" form.
// ------------------------------------------------------------------
Boolean LeagueOrderFormEvtHandler(EventPtr event) {
  Boolean handled = false;

  switch (event->eType) {

   // form is opened
    case frmOpenEvent:
      InitLeagueOrderForm();
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;

   // form is closed
    case frmCloseEvent:
      FreeLeagueOrderForm();
      break;

   // a button was pressed
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {

        case btnID_moveUp:
          MoveLeague(-1);
          handled = true;
          break;

        case btnID_moveDown:
          MoveLeague(1);
          handled = true;
          break;

        // close -> save order and return to previous form
        case btnID_close:
          FreeLeagueOrderForm();
          FrmReturnToForm(0);
          SaveSettings(); // the league order is stored in the app info block
          ShowLeague(0, 0, MODE_TABLE, true, true);
          handled = true;
          break;
      }
      break;

    default:
  }

  return handled;
}


// ------------------------------------------------------------------
//
// Today's matches
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Finds matches in the available leagues for a given date. The match strings
// are placed in 'matchStrings' and the number of strings is returned.
// ------------------------------------------------------------------
Int16 GetTodaysMatches(DateType* date, Char** matchStrings) {
  Int16   i, j;
  Int16   stringCount;
  Boolean isStringsLeft = true;
  UInt16  cardNo;
  LocalID localId;
  League  league;
  Int16   matchCount;
  Int16   matchesForDate[MAX_TEAMS];
  LeagueInfo* tempLeagueInfo;

  // remember the currently open database
  cardNo  = GState.currentLeagueInfo->cardNo;
  localId = GState.currentLeagueInfo->localID;

  // loop through all the leagues - in a quite unefficient way :-(
  // the outmost for loop iterates through all the league names to make
  // sure the leagues appear in the same order as in the league drop-down.
  // the outer do loop iterates through all the league names and the
  // inner do loop iterates through all the seasons of a league.
  stringCount = 0;

  for (j = 0; j < GState.leagueNameCount; j++) {

    tempLeagueInfo = GLeagueInfo;

    do {
      // check if should show this league now (so the order is correct)
      if (StrCompare(GLeagueNameLabels[j], tempLeagueInfo->name) == 0) {

        LeagueInfo* tempLeagueInfo2 = tempLeagueInfo;

        do {

          // open the league
          CloseCurrentDatabase();
          OpenDatabase(tempLeagueInfo2->cardNo, tempLeagueInfo2->localID);
          league = LeagueGetCurrent();

          // get matches for the date
          matchCount = LeagueGetMatchesForDate(&league, date, matchesForDate);
          if (matchCount > 0) {
            // bail out if we've ran out of string space
            if (stringCount+2 >= MAX_TODAYS_MATCHES) {
              isStringsLeft = false;
              break;
            }
            // if matches were found -> add the league name as a header...
            matchStrings[stringCount] = MemPtrNew(35);
            StrCopy(matchStrings[stringCount++], "");
            matchStrings[stringCount] = MemPtrNew(35);
            StrCopy(matchStrings[stringCount++], tempLeagueInfo->name);
            matchStrings[stringCount] = MemPtrNew(35);
            StrCopy(matchStrings[stringCount++], "===============");
            // ...and then all the matches for the date
            for (i = 0; i < matchCount; i++) {
              // bail out if we've ran out of string space
              if (stringCount >= MAX_TODAYS_MATCHES) {
                isStringsLeft = false;
                break;
              }
              matchStrings[stringCount] = MemPtrNew(35);
              LeagueGetMatchStringNoDate(&league, matchesForDate[i], matchStrings[stringCount++]);
            }
          }

          // go to next season
          tempLeagueInfo2 = tempLeagueInfo2->nextSeason;
        } while (tempLeagueInfo2 != NULL && isStringsLeft);

      }

      // go to next league
      tempLeagueInfo = tempLeagueInfo->nextLeague;
    } while (tempLeagueInfo != NULL && isStringsLeft);

    // exit if we're out of strings
    if (!isStringsLeft) break;

  } // for loop

  // restore the original database
  CloseCurrentDatabase();
  OpenDatabase(cardNo, localId);

  return stringCount;
}


// ------------------------------------------------------------------
// Frees memory for strings.
// ------------------------------------------------------------------
void FreeTodaysString() {
  Int16 i;

  // unlock the pointers to the strings
  for(i = 0; i < GStringCount; i++) {
    MemPtrFree(GTodayStrings[i]);
  }
}


// ------------------------------------------------------------------
// Initializes "Todays matches" form.
// ------------------------------------------------------------------
void InitTodaysMatchesForm() {
  GStringCount = 0;
  // get todays date
  DateSecondsToDate(TimGetSeconds(), &GDate);
}


// ------------------------------------------------------------------
// Frees "Todays matches" form.
// ------------------------------------------------------------------
void FreeTodaysMatchesForm() {
  FreeTodaysString();
}


// ------------------------------------------------------------------
// Redraws the form
// ------------------------------------------------------------------
void UpdateTodaysMatchesForm() {
  Char    buf[30];
  FormPtr form = FrmGetActiveForm();
  ListPtr matchList;

  // free previously allocated memory
  FreeTodaysString();

  // get the match strings to display,
  // memory is allocated for the strings by GetTodaysMatches()
  GStringCount = GetTodaysMatches(&GDate, GTodayStrings);

  // set matches for the list control
  matchList = (ListPtr)UIGetObjectPtr(listID_fixtures);
  LstSetListChoices(matchList, GTodayStrings, GStringCount);
  LstSetSelection(matchList, -1);
  if (GStringCount > 0) {
    LstSetTopItem(matchList, 0);
  }

  // date control
  StrPrintF(buf, TEXT_TODAYS_MATCHES_DATE,
            GWeekDays[DayOfWeek(GDate.month, GDate.day, GDate.year+1904)], GDate.day, GDate.month);
  UISetControlLabel(form, selectTrigID_date, buf);
}


// ------------------------------------------------------------------
// Eventhandler for "Todays matches" form.
// ------------------------------------------------------------------
Boolean TodaysMatchesFormEvtHandler(EventPtr event) {
  Boolean handled      = false;
  Boolean updateScreen = false;
  Int16   year, month, day;
  ListPtr matchList;

  switch (event->eType) {

    // form is opened
    case frmOpenEvent:
      InitTodaysMatchesForm();
      updateScreen = true;
      handled = true;
      break;

   // form is closed
    case frmCloseEvent:
      FreeTodaysMatchesForm();
      break;

   // form need to be updated
    case frmUpdateEvent:
      updateScreen = true;
      handled = true;
      break;

   // one of the scrollbuttons was pressed
    case keyDownEvent:
      switch(event->data.keyDown.chr) {
        case pageUpChr :
          matchList = (ListPtr)UIGetObjectPtr(listID_fixtures);
          LstScrollList(matchList, winUp, 8);
          handled = true;
          break;
        case pageDownChr:
          matchList = (ListPtr)UIGetObjectPtr(listID_fixtures);
          LstScrollList(matchList, winDown, 8);
          handled = true;
          break;
      }
      break;

    // a button was pressed
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {

        // close the form
        case btnID_close:
          FreeTodaysMatchesForm();
          FrmReturnToForm(0);
          handled = true;
          break;
        // go to previous day
        case btnID_dayBack:
          DateAdjust(&GDate, -1);
          handled = true;
          updateScreen = true;
          break;
        // go to next day
        case btnID_dayFwd:
          DateAdjust(&GDate, 1);
          handled = true;
          updateScreen = true;
          break;
        // date-control tapped
        case selectTrigID_date:
          year  = GDate.year + 1904;
          month = GDate.month;
          day   = GDate.day;
          SelectDay(selectDayByDay, &month, &day, &year, TEXT_MATCH_SELECT_MATCH_DAY);
          GDate.year  = year - 1904;
          GDate.month = month;
          GDate.day   = day;
          updateScreen = true;
          break;
      }
      break;

    default:
  }

  //  redraw the screen if necessary
  if(updateScreen) {
    UpdateTodaysMatchesForm();
    FrmDrawForm(FrmGetActiveForm());
  }

  return handled;
}
