/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "LeagueDialogs.h"
#include "Text.h"
#include <HostControl.h>


// ------------------------------------------------------------------
// Global variables
// ------------------------------------------------------------------

// The state of the application
AppState GState;

// The currently open league database
DmOpenRef GCurrentDB = NULL;

// Info about the league databases
LeagueInfo* GLeagueInfo = NULL;

// All the names and seasons of the leagues (used in drop-downs)
Char** GLeagueNameLabels = NULL;
Char** GSeasonLabels     = NULL;

Char GChangeOrder[] = TEXT_MAIN_EDIT_ORDER;


// ------------------------------------------------------------------
// Forward declarations
// ------------------------------------------------------------------
void      PrepareSeasonLabels(Int16 leagueNameIndex);
DmOpenRef GetSettingsDatabase(Boolean doCreate);



// ------------------------------------------------------------------
// Functions
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Frees all memory allocated to league info (GLeagueInfo).
// ------------------------------------------------------------------
void FreeLeagueInfoMem() {
  LeagueInfo* leagueInfo;
  LeagueInfo* leagueInfoNextSeason;
  LeagueInfo* leagueInfoRemove;

  // loop through all the leagues
  leagueInfo = GLeagueInfo;
  while (leagueInfo != NULL) {
    // loop through all the seasons of a league
    leagueInfoNextSeason = (LeagueInfo*)leagueInfo->nextSeason;
    while (leagueInfoNextSeason != NULL) {
      leagueInfoRemove = leagueInfoNextSeason;
      leagueInfoNextSeason = (LeagueInfo*)leagueInfoNextSeason->nextSeason;
      MemPtrFree(leagueInfoRemove);
    }
    leagueInfoRemove = leagueInfo;
    leagueInfo = (LeagueInfo*)leagueInfo->nextLeague;
    MemPtrFree(leagueInfoRemove);
  }

  GLeagueInfo = NULL;
}


// ------------------------------------------------------------------
// Frees memory for league name labels, used in drop-downs
// ------------------------------------------------------------------
void FreeLeagueNameLabels() {
  if (GLeagueNameLabels != NULL) {
    MemPtrFree(GLeagueNameLabels);
  }
}


// ------------------------------------------------------------------
// Frees memory for season labels, used in drop-downs
// ------------------------------------------------------------------
void FreeSeasonLabels() {
  if (GSeasonLabels != NULL) {
    Int16 i;
    for(i = 0; i < GState.seasonCount; i++) {
      MemPtrFree(GSeasonLabels[i]);
    }
    MemPtrFree(GSeasonLabels);
  }
}


// ------------------------------------------------------------------
// Inserts a LeagueInfo structure into the global reference. LeagueInfo
// has two pointers: nextLeague and nextSeason. Seasons are sorted with
// the most recent at the top. Only top level leagues (the most recent
// season) uses the nextLeague pointer.
// ------------------------------------------------------------------
void InsertLeagueInfo(LeagueInfo* newLeagueInfo) {

  // if this is the first league just asign it to GLeagueInfo
  if (GLeagueInfo == NULL) {
    GLeagueInfo = newLeagueInfo;
  }

  // check if it is the first league in the list
  else if (StrCompare(GLeagueInfo->name, newLeagueInfo->name) == 0) {
    if (newLeagueInfo->season > GLeagueInfo->season) {
      // this is the most recent league -> put it at the top
      newLeagueInfo->nextLeague = GLeagueInfo->nextLeague;
      newLeagueInfo->nextSeason = GLeagueInfo;
      // since this is the first league we point GLeagueInfo to it
      GLeagueInfo = newLeagueInfo;
    }
    else {
      // loop through the seasons to find the correct position
      LeagueInfo* tempLeagueInfo = GLeagueInfo;
      while (tempLeagueInfo->nextSeason != NULL) {
        if (newLeagueInfo->season > tempLeagueInfo->nextSeason->season) {
          newLeagueInfo->nextSeason = tempLeagueInfo->nextSeason;
          break;
          // tempLeagueInfo->nextSeason will be assigned to newLeagueInfo below
        }
        tempLeagueInfo = tempLeagueInfo->nextSeason;
      }
      tempLeagueInfo->nextSeason = newLeagueInfo;
    }
  }

  // it's not first league -> we need to loop to find the league
  else {
    // loop through the leagues
    LeagueInfo* tempLeagueInfo = GLeagueInfo;
    while (tempLeagueInfo->nextLeague != NULL) {
      if (StrCompare(newLeagueInfo->name, tempLeagueInfo->nextLeague->name) == 0) {
        if (newLeagueInfo->season > tempLeagueInfo->nextLeague->season) {
          // this is the most recent league -> put it at the top
          newLeagueInfo->nextLeague  = tempLeagueInfo->nextLeague->nextLeague;
          newLeagueInfo->nextSeason  = tempLeagueInfo->nextLeague;
          tempLeagueInfo->nextLeague = newLeagueInfo;
        }
        else {
          // loop through the seasons to find the correct position
          LeagueInfo* tempLeagueInfo2 = tempLeagueInfo->nextLeague;
          while (tempLeagueInfo2->nextSeason != NULL) {
            if (newLeagueInfo->season > tempLeagueInfo2->nextSeason->season) {
              newLeagueInfo->nextSeason = tempLeagueInfo2->nextSeason;
              break;
              // tempLeagueInfo2->nextSeason will be assigned to newLeagueInfo below
            }
            tempLeagueInfo2 = tempLeagueInfo2->nextSeason;
          }
          tempLeagueInfo2->nextSeason = newLeagueInfo;
        }
        // the league has been inserted -> exit functions
        return;
      }
      tempLeagueInfo = tempLeagueInfo->nextLeague;
    }

    // if we arrive here the league wasn't found -> append it after the last league
    tempLeagueInfo->nextLeague = newLeagueInfo;
  }

}


// ------------------------------------------------------------------
// Returns the LeagueInfo structure that matches the name
// of 'leagueNameIndex' and the season of 'seasonIndex'.
// ------------------------------------------------------------------
LeagueInfo* FindLeagueInfo(Int16 leagueNameIndex, Int16 seasonIndex) {
  Int16       i;
  LeagueInfo* tempLeagueInfo = GLeagueInfo;

  // first we loop to find the correct league
  do {
    if ((StrCompare(GLeagueNameLabels[leagueNameIndex], tempLeagueInfo->name) == 0)) {
      break;
    }
    tempLeagueInfo = tempLeagueInfo->nextLeague;
  } while (tempLeagueInfo != NULL);

  // secondly we loop to find the season
  for (i = 0; i < seasonIndex; i++) {
    tempLeagueInfo = tempLeagueInfo->nextSeason;
  }

  return tempLeagueInfo;
}


// ------------------------------------------------------------------
// Initializes the top toolbar of the form, ie the mode buttons and league selector.
// This top part is equal for all the mode forms.
// ------------------------------------------------------------------
void InitTop(Int8 mode) {
  UInt16     btnID = 0;
  ControlPtr compareCtl;
  ListPtr    leagueList;
  ListPtr    seasonList;

  switch(mode) {
    case MODE_TABLE:   btnID = btnID_table;   break;
    case MODE_MATCHES: btnID = btnID_matches; break;
    case MODE_TEAM:    btnID = btnID_team;    break;
    case MODE_LEAGUE:  btnID = btnID_league;  break;
  }
  // make the correct mode button selected.
  compareCtl = (ControlPtr)UIGetObjectPtr(btnID);
  CtlSetValue(compareCtl, 1);

  // populate the league drop-down
  leagueList = (ListPtr)UIGetObjectPtr(popListID_league);
  LstSetListChoices(leagueList, GLeagueNameLabels, GState.leagueNameCount+1);
  LstSetHeight(leagueList, min(GState.leagueNameCount+1, 14));
  // select the current league
  UISelectDropDown(popListID_league, popTrigID_league, GState.currentLeagueNameIndex);

  // populate the season drop-down
  seasonList = (ListPtr)UIGetObjectPtr(popListID_season);
  LstSetListChoices(seasonList, GSeasonLabels, GState.seasonCount);
  LstSetHeight(seasonList, min(GState.seasonCount, 14));
  // select the current season
  UISelectDropDown(popListID_season, popTrigID_season, GState.currentSeasonIndex);
}


// ------------------------------------------------------------------
// Sets default state for the current league.
// ------------------------------------------------------------------
void InitLeague(Int8 startupMode) {
  League league = LeagueGetCurrent();
 // app settings
  GState.mode = startupMode;
 // table form settings
  GState.Table.matches    = 0;
  GState.Table.rounds     = 0;
  GState.Table.roundCount = 5;
  GState.Table.view       = 0;
  GState.Table.scrollPos  = 0;
 // match form settings
  GState.Match.round     = LeagueGetCurrentRound(&league);
  GState.Match.scrollPos = 0;
 // team form settings
  GState.Team.team      = 0;
  GState.Team.matches   = 0;
  GState.Team.mode      = 0;
  GState.Team.scrollPos = 0;
}


// ------------------------------------------------------------------
// Switches to the specified league and mode.
// ------------------------------------------------------------------
void ShowLeague(Int8 leagueNameIndex, Int8 seasonIndex, Int8 startupMode, Boolean initState, Boolean rePopulateSeason) {
  UInt16 formID = 0;

  // sanity check of leagueNameIndex
  if (leagueNameIndex >= GState.leagueNameCount) {
    leagueNameIndex = 0;
  }

  if (rePopulateSeason) {
    PrepareSeasonLabels(leagueNameIndex);
  }

  // sanity check of seasonIndex
  if (seasonIndex >= GState.seasonCount) {
    seasonIndex = 0;
  }

  // close the current db and open the new one
  CloseCurrentDatabase();
  GState.currentLeagueInfo = FindLeagueInfo(leagueNameIndex, seasonIndex);
  OpenDatabase(GState.currentLeagueInfo->cardNo, GState.currentLeagueInfo->localID);
  GState.currentLeagueNameIndex = leagueNameIndex;
  GState.currentSeasonIndex     = seasonIndex;

  // initialize the league...
  if (initState) {
    InitLeague(startupMode);
  }

  // ...and show the desired form
  switch(startupMode) {
    case MODE_TABLE:   formID = formID_table; break;
    case MODE_MATCHES: formID = formID_matches; break;
    case MODE_TEAM:    formID = formID_team; break;
    case MODE_LEAGUE:  formID = formID_league; break;
  }

  FrmGotoForm(formID);
}


// ------------------------------------------------------------------
// Fills the leagueInfo structure with data about the league.
// ------------------------------------------------------------------
void FillLeagueInfo(LocalID dbId, UInt16 cardNo, LeagueInfo* leagueInfo) {
  Int8      dbVersion;
  DmOpenRef db        = DmOpenDatabase(cardNo, dbId, dmModeReadWrite);
  MemHandle recHandle = DmQueryRecord(db, 0);
  UInt8*    recPtr    = (UInt8*)MemHandleLock(recHandle);

  // get the first record
  MemMove(&dbVersion, recPtr+13, 1);

  // if it's an old database we need to upgrade it
  if (dbVersion < 3) {
    MemHandleUnlock(recHandle);
    LeagueUpgradeDatabase(db);
    // re-open the first record
    recHandle = DmQueryRecord(db, 0);
    recPtr    = (UInt8*)MemHandleLock(recHandle);
  }

  // get the league name and season
  MemMove(leagueInfo->name, recPtr+14, 17);
  MemMove(&(leagueInfo->season), recPtr+31, 2);

  MemHandleUnlock(recHandle);
  DmCloseDatabase(db);

  // if season is negative the league is split over two years
  leagueInfo->splitYear  = (leagueInfo->season < 0);
  leagueInfo->season     = abs(leagueInfo->season);
  leagueInfo->localID    = dbId;
  leagueInfo->cardNo     = cardNo;
  leagueInfo->nextLeague = NULL;
  leagueInfo->nextSeason = NULL;
}


// ------------------------------------------------------------------
// Adds league info to the global data structures.
// ------------------------------------------------------------------
void AddLeagueInfo(LocalID dbId, UInt16 cardNo) {
  LeagueInfo* leagueInfoNew;

  // allocate memory for the league info structure
  leagueInfoNew = MemPtrNew(sizeof(LeagueInfo));
  // fill the structure with data
  FillLeagueInfo(dbId, cardNo, leagueInfoNew);
  // insert it to the global reference
  InsertLeagueInfo(leagueInfoNew);

  GState.leagueCount++;
}


// ------------------------------------------------------------------
// Searches for league databases. The localID and cardNo of each found
// league is stored in a globally available data structure.
// ------------------------------------------------------------------
void SearchForDatabases() {
  Err               err;
  DmSearchStateType state;
  UInt16            cardNo;
  LocalID           currentDbId = 0;

  // look for the first league database
  err = DmGetNextDatabaseByTypeCreator(true, &state, 'Data', 'mbTS', false, &cardNo, &currentDbId);

  while (!err && currentDbId) {
    Char dbName[32];

    DmDatabaseInfo(cardNo, currentDbId, dbName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

    if (StrCompare(dbName, "mbTS_settings") != 0) {
      // store league info (name, season, database location) in a global data structure
      AddLeagueInfo(currentDbId, cardNo);
    }

    // search for the next db
    err = DmGetNextDatabaseByTypeCreator(false, &state, 'Data', 'mbTS', false, &cardNo, &currentDbId);
  }

}


// ------------------------------------------------------------------
// Creates labels for the season drop-down.
// ------------------------------------------------------------------
void PrepareSeasonLabels(Int16 leagueNameIndex) {
  LeagueInfo* tempLeagueInfo = GLeagueInfo;
  Int16       season;

  // free memory of previous labels
  FreeSeasonLabels();
  GState.seasonCount = 0;

  // allocate memory for the label pointers
  // will probably allocate to much memory but it is resized later on
  GSeasonLabels = (Char **)MemPtrNew(GState.leagueCount * sizeof(Char*));

  // loop through the leagues to find the correct league
  do {
    if (StrCompare(tempLeagueInfo->name, GLeagueNameLabels[leagueNameIndex]) == 0) {
      // go through all the seasons and a label for each one
      // the leagues are already sorted in the way we want to display them
      LeagueInfo *tempSeason = tempLeagueInfo;
      do {
        // allocate memory for the label
        GSeasonLabels[GState.seasonCount] = (Char *)MemPtrNew(8);
        // we create slighlty different labels depending on if the league
        // is split over two years or if it's played in a single year
        if (tempSeason->splitYear) {
          season = tempSeason->season % 100;
          StrPrintF(GSeasonLabels[GState.seasonCount], "%2d/%2d", season, (season+1)%100);
          if (season < 9) GSeasonLabels[GState.seasonCount][0] = '0';
          if ((season+1)%100 < 9) GSeasonLabels[GState.seasonCount][3] = '0';
        }
        else {
          StrPrintF(GSeasonLabels[GState.seasonCount], "%d", tempSeason->season);
        }

        GState.seasonCount++;

        tempSeason = tempSeason->nextSeason;
      } while (tempSeason != NULL);
    }

    tempLeagueInfo = tempLeagueInfo->nextLeague;
  } while (tempLeagueInfo != NULL);

  // resize the memory to the actual size needed
  MemPtrResize(GSeasonLabels, GState.seasonCount * sizeof(Char*));
}


// ------------------------------------------------------------------
// Creates labels for the league name drop-down.
// ------------------------------------------------------------------
void PrepareLeagueNameLabels() {
  Int16         i;
  LeagueInfo*   tempLeagueInfo;
  DmOpenRef     dbRef;

  // free memory of previous labels
  FreeLeagueNameLabels();
  GState.leagueNameCount = 0;

  // allocate memory for the label pointers
  // will probably allocate to much memory but it is resized later on
  GLeagueNameLabels = (Char**) MemPtrNew((GState.leagueCount+1)*sizeof(Char*));

  // first, we look in the settings database so we get the order of the leagues correct
  dbRef = GetSettingsDatabase(false);

  if (dbRef) {
    Int8      leagueNameCount;
    Char      tempLeagueName[LEAGUENAME_SIZE];
    MemHandle recHandle = DmGetRecord(dbRef, 1);
    void*     recPtr    = (void*)MemHandleLock(recHandle);

    // get the league name count
    MemMove(&leagueNameCount, recPtr, 1);

    // loop through the names stored in the settings database
    for (i = 0; i < leagueNameCount; i++) {

      MemMove(tempLeagueName, recPtr + 1 + (i*LEAGUENAME_SIZE), LEAGUENAME_SIZE);
      tempLeagueInfo = GLeagueInfo;
      do {
        if (StrCompare(tempLeagueName, tempLeagueInfo->name) == 0) {
          // set the label pointer to the name stored in LeagueInfo
          GLeagueNameLabels[GState.leagueNameCount++] = tempLeagueInfo->name;
          break;
        }
        tempLeagueInfo = tempLeagueInfo->nextLeague;
      } while (tempLeagueInfo != NULL);

    }

    MemHandleUnlock(recHandle);
    DmReleaseRecord(dbRef, 1, false);
    DmCloseDatabase(dbRef);
  }

  // loop through all the leagues to find leagues that were not included in the settings db
  tempLeagueInfo = GLeagueInfo;
  do {
    // loop to avoid duplicates
    for (i = 0; i < GState.leagueNameCount; i++) {
      if (StrCompare(GLeagueNameLabels[i], tempLeagueInfo->name) == 0) {
        break;
      }
    }
    if (i == GState.leagueNameCount) {
      GLeagueNameLabels[GState.leagueNameCount++] = tempLeagueInfo->name;
    }
    tempLeagueInfo = tempLeagueInfo->nextLeague;
  } while (tempLeagueInfo != NULL);

  // add the "Edit order..." option
  GLeagueNameLabels[GState.leagueNameCount] = GChangeOrder;

  // resize the memory to the actual size needed
  MemPtrResize(GLeagueNameLabels, (GState.leagueNameCount+1)*sizeof(Char*));
}


// ------------------------------------------------------------------
// Finds league databases and initializes some global structures.
// ------------------------------------------------------------------
void InitDatabases() {
  FreeLeagueInfoMem();
  GState.leagueCount = 0;
  SearchForDatabases();
  if (GState.leagueCount > 0) {
    PrepareLeagueNameLabels();
  }
}


// ------------------------------------------------------------------
// Copy the current state (GState) to the supplied structure
// (which is later stored in a database).
// ------------------------------------------------------------------
void FillPersistentAppState(PersistentAppState* appStatePtr) {

  appStatePtr->version = 1;
  StrCopy(appStatePtr->leagueName, GLeagueNameLabels[GState.currentLeagueNameIndex]);
  appStatePtr->seasonIndex     = GState.currentSeasonIndex;
  appStatePtr->mode            = GState.mode;
  appStatePtr->tableMatches    = GState.Table.matches;
  appStatePtr->tableRounds     = GState.Table.rounds;
  appStatePtr->tableRoundCount = GState.Table.roundCount;
  appStatePtr->tableView       = GState.Table.view;
  appStatePtr->tableScrollPos  = GState.Table.scrollPos;
  appStatePtr->matchRound      = GState.Match.round;
  appStatePtr->matchScrollPos  = GState.Match.scrollPos;
  appStatePtr->teamTeam        = GState.Team.team;
  appStatePtr->teamMatches     = GState.Team.matches;
  appStatePtr->teamMode        = GState.Team.mode;
  appStatePtr->teamScrollPos   = GState.Team.scrollPos;
}


// ------------------------------------------------------------------
// Copies data from the supplied data structure (which has been read
// from a database) to the the current state structure GState.
// Returns false if the current league isn't found (ie the league has
// been deleted).
// ------------------------------------------------------------------
Boolean InitAppState(PersistentAppState* appStatePtr) {
  Int16 i = 0;

  // find the league that should be the current league
  while ((i < GState.leagueNameCount) && StrCompare(GLeagueNameLabels[i], appStatePtr->leagueName)) {
    i++;
  }

  // no league found -> exit from function
  if (i == GState.leagueNameCount) {
    return false;
  }

  GState.currentLeagueNameIndex = i;
  GState.currentSeasonIndex = appStatePtr->seasonIndex;
  GState.mode               = appStatePtr->mode;
  GState.Table.matches      = appStatePtr->tableMatches;
  GState.Table.rounds       = appStatePtr->tableRounds;
  GState.Table.roundCount   = appStatePtr->tableRoundCount;
  GState.Table.view         = appStatePtr->tableView;
  GState.Table.scrollPos    = appStatePtr->tableScrollPos;
  GState.Match.round        = appStatePtr->matchRound;
  GState.Match.scrollPos    = appStatePtr->matchScrollPos;
  GState.Team.team          = appStatePtr->teamTeam;
  GState.Team.matches       = appStatePtr->teamMatches;
  GState.Team.mode          = appStatePtr->teamMode;
  GState.Team.scrollPos     = appStatePtr->teamScrollPos;

  return true;
}


// ------------------------------------------------------------------
// Opens the settings database. If the database isn't found and
// doCreate is true the database is created.
// The returned dbRef can be NULL if the database couldn't be created.
// ------------------------------------------------------------------
DmOpenRef GetSettingsDatabase(Boolean doCreate) {
  LocalID   dbId;
  DmOpenRef dbRef = NULL;

  // look for the settings db
  dbId = DmFindDatabase(0, "mbTS_settings");

  // if it's found -> open it
  if (dbId != 0) {
    // make sure it's the correct creator
    UInt32 creator;
    DmDatabaseInfo(0, dbId, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &creator);
    if (creator == 'mbTS') {
      dbRef = DmOpenDatabase(0, dbId, dmModeReadWrite);
    }
  }

  // if it's not found and doCreate is true -> create the settings db
  else if(doCreate) {

    // create db and open it
    DmCreateDatabase(0, "mbTS_settings", 'mbTS', 'Data', false);
    dbId  = DmFindDatabase(0, "mbTS_settings");

    if (dbId) {
      // set backup flag
      UInt16 dbAttrs = dmHdrAttrBackup;
      DmSetDatabaseInfo(0, dbId, NULL, &dbAttrs, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

      dbRef = DmOpenDatabase(0, dbId, dmModeReadWrite);

      if (dbRef) {
        UInt16 recIndex;

        // create the first record which is used for state info
        recIndex = 0;
        DmNewRecord(dbRef, &recIndex, sizeof(PersistentAppState));
        DmReleaseRecord(dbRef, recIndex, true);

        // create the second record which is for league order in the drop-down
        recIndex = 1;
        DmNewRecord(dbRef, &recIndex, 1);
        DmReleaseRecord(dbRef, recIndex, true);
      }
    }
  }

  return dbRef;
}


// ------------------------------------------------------------------
// Saves the current state of the application to the settings database.
// Called when the user exits from the application and when the
// league order is changed.
// ------------------------------------------------------------------
void SaveSettings() {

  if (GState.leagueCount > 0) {

    DmOpenRef dbRef = GetSettingsDatabase(true);

    if (dbRef) {
      Int16     i;
      MemHandle recHandle;
      void*     recPtr;
      PersistentAppState appState;

      // get the current app state
      FillPersistentAppState(&appState);

      // write app state to first record
      recHandle = DmGetRecord(dbRef, 0);
      recPtr    = (void*)MemHandleLock(recHandle);
      DmWrite(recPtr, 0, &appState, sizeof(PersistentAppState));
      MemHandleUnlock(recHandle);
      DmReleaseRecord(dbRef, 0, true);

      // the second record contains the league order used in drop-down
      DmResizeRecord(dbRef, 1, 1 + (GState.leagueNameCount*LEAGUENAME_SIZE));
      recHandle = DmGetRecord(dbRef, 1);
      recPtr    = (void*)MemHandleLock(recHandle);
      DmWrite(recPtr, 0, &GState.leagueNameCount, 1);

      for (i = 0; i < GState.leagueNameCount; i++) {
        DmWrite(recPtr, 1 + (i*LEAGUENAME_SIZE), GLeagueNameLabels[i], LEAGUENAME_SIZE);
      }

      MemHandleUnlock(recHandle);
      DmReleaseRecord(dbRef, 1, true);

      // close the db
      DmCloseDatabase(dbRef);
    }
  }
}


// ------------------------------------------------------------------
// Reads saved state from the settings database. Called when
// the application starts.
// ------------------------------------------------------------------
Boolean ReadAppState() {
  PersistentAppState appState;
  Boolean            appStateFound = false;
  DmOpenRef          dbRef;

  // open the settings database
  dbRef = GetSettingsDatabase(false);

  // get the app state from the first record
  if (dbRef) {
    MemHandle recHandle = DmGetRecord(dbRef, 0);
    void*     recPtr    = (void*)MemHandleLock(recHandle);

    MemMove(&appState, recPtr, sizeof(PersistentAppState));

    MemHandleUnlock(recHandle);
    DmReleaseRecord(dbRef, 0, false);
    DmCloseDatabase(dbRef);

    appStateFound = InitAppState(&appState);
  }

  return appStateFound;
}


// ------------------------------------------------------------------
// Event handler for the "no league"-form.
// ------------------------------------------------------------------
Boolean NoLeagueFormEvtHandler(EventPtr event) {
  FormPtr form    = FrmGetActiveForm();
  Boolean handled = false;

  switch (event->eType) {
    // form is opened
    case frmOpenEvent:
      FrmDrawForm(form);
      handled = true;
      break;

    default:
  }

  return handled;
}


// ------------------------------------------------------------------
// The application event handler. Used for launching the different forms.
// ------------------------------------------------------------------
Boolean ApplicationHandleEvent(EventPtr event) {
  Boolean handled = false;

  if(event->eType == frmLoadEvent) {
    UInt16  formId = event->data.frmLoad.formID;
    FormPtr form   = FrmInitForm(formId);
    FrmSetActiveForm(form);

    switch(formId) {
      case formID_noleague:
        FrmSetEventHandler(form, (FormEventHandlerPtr)NoLeagueFormEvtHandler);
        handled = true;
        break;
      case formID_table:
        FrmSetEventHandler(form, (FormEventHandlerPtr)TableFormEvtHandler);
        handled = true;
        break;
      case formID_matches:
        FrmSetEventHandler(form, (FormEventHandlerPtr)MatchFormEvtHandler);
        handled = true;
        break;
      case formID_team:
        FrmSetEventHandler(form, (FormEventHandlerPtr)TeamFormEvtHandler);
        handled = true;
        break;
      case formID_league:
        FrmSetEventHandler(form, (FormEventHandlerPtr)LeagueFormEvtHandler);
        handled = true;
        break;
      case formID_createLeague:
        FrmSetEventHandler(form, (FormEventHandlerPtr)CreateLeagueFormEvtHandler);
        handled = true;
        break;
      case formID_leagueOrder:
        FrmSetEventHandler(form, (FormEventHandlerPtr)LeagueOrderFormEvtHandler);
        handled = true;
        break;
      case formID_editLeague:
        FrmSetEventHandler(form, (FormEventHandlerPtr)EditLeagueFormEvtHandler);
        handled = true;
        break;
      case formID_editTeams:
        FrmSetEventHandler(form, (FormEventHandlerPtr)EditTeamsFormEvtHandler);
        handled = true;
        break;
      case formID_editFixtures:
        FrmSetEventHandler(form, (FormEventHandlerPtr)EditFixturesFormEvtHandler);
        handled = true;
        break;
      case formID_todaysMatches:
        FrmSetEventHandler(form, (FormEventHandlerPtr)TodaysMatchesFormEvtHandler);
        handled = true;
        break;
      case formID_selectRound:
        FrmSetEventHandler(form, (FormEventHandlerPtr)SelectRoundFormEvtHandler);
        handled = true;
        break;
      case formID_about:
        FrmSetEventHandler(form, (FormEventHandlerPtr)AboutFormEvtHandler);
        handled = true;
        break;
    }
  }

  if(event->eType == menuEvent) {
    UInt16 buttonIndex;
    UInt32 romversion;
    Char   nameBuf[32];
    League league;

    // send an update event to the match form so it
    // saves any unsaved results before we continue with
    // the menu handling
    FrmUpdateForm(formID_matches, 0);

    switch (event->data.menu.itemID) {
      case menuitemID_main_about:
      case menuitemID_noleague_about:
        FrmPopupForm(formID_about);
        break;
      case menuitemID_main_today:
        // display "Todays matches" dialog
        FrmPopupForm(formID_todaysMatches);
        break;
      case menuitemID_main_create:
      case menuitemID_noleague_create:
        // display "Create league" dialog
        FrmPopupForm(formID_createLeague);
        break;
      case menuitemID_main_delete:
        // display "Really delete?" confirmation
        league = LeagueGetCurrent();
        CreateLeagueDbName(&league, nameBuf);
        //GLeagueNameLabels[GState.currentLeagueNameIndex]
        buttonIndex = FrmCustomAlert(alertID_deleteLeague, nameBuf, NULL, NULL);
        if(buttonIndex == 0) { // yes was clicked
          // delete the current db
          DeleteCurrentDatabase();
          // show table form for the first available database...
          if(GState.leagueCount > 0) {
            ShowLeague(0, 0, MODE_TABLE, true, true);
          }
          // ...otherwise show "no league form"
          else {
            FrmGotoForm(formID_noleague);
          }
        }
        break;
      case menuitemID_main_beam:
        // check that we're on an OS version that supports beaming
        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);
        if(romversion >= 0x03003000) {
          BeamCurrentDatabase();
        }
        else {
          FrmCustomAlert(alertID_errGeneral, TEXT_MAIN_BEAMING_NOT_SUPPORTED, NULL, NULL);
        }
        break;
      case menuitemID_main_league:
        // display "Edit league" dialog
        FrmPopupForm(formID_editLeague);
        break;
      case menuitemID_main_teams:
        // display "Edit teams" dialog
        FrmPopupForm(formID_editTeams);
        break;
      case menuitemID_main_fixtures:
        // display "Edit fixtures" dialog
        FrmPopupForm(formID_editFixtures);
        break;
    }
    //handled = true;
  }

  return handled;
}


// ------------------------------------------------------------------
// The main event loop of the application.
// ------------------------------------------------------------------
void EventLoop() {
  EventType event;
  Err       err;

  do {
    EvtGetEvent(&event, 200);

    if(SysHandleEvent(&event))
      continue;
    if(MenuHandleEvent(NULL, &event, &err))
      continue;
    if(ApplicationHandleEvent(&event))
      continue;
    FrmDispatchEvent(&event);
  } while(event.eType != appStopEvent);
}


// ------------------------------------------------------------------
// Cleans up when application is about to finish.
// ------------------------------------------------------------------
void StopApplication() {
  FrmCloseAllForms();
  SaveSettings();
  CloseCurrentDatabase();

  FreeLeagueNameLabels();
  FreeSeasonLabels();
  FreeLeagueInfoMem();
}


// ------------------------------------------------------------------
// Initializes the app by looking for league databases and
// then launching the main form.
// ------------------------------------------------------------------
void StartApplication() {
  Boolean appStateFound;
  Int8 leagueNameIndex = 0;
  Int8 seasonIndex     = 0;
  Int8 mode            = MODE_TABLE;

  InitDatabases();
  // read the saved app state and start the application
  // where it was finished
  if(GState.leagueCount > 0) {
    appStateFound = ReadAppState();
    if (appStateFound) {
      leagueNameIndex = GState.currentLeagueNameIndex;
      seasonIndex     = GState.currentSeasonIndex;
      mode = GState.mode;
    }
    ShowLeague(leagueNameIndex, seasonIndex, mode, !appStateFound, true);
  }
  // or show "no league form" if no leagues are available
  else {
    FrmGotoForm(formID_noleague);
  }
}

/*
void TraceMem() {
  UInt16 i;
  UInt16 numHeaps;
  UInt32 heapSize;
  UInt32 freeBytes;
  UInt32 maxSize;

  numHeaps = MemNumHeaps(0);
  HostTraceOutputTL(appErrorClass, "num cards %hu", MemNumCards());
  HostTraceOutputTL(appErrorClass, "num heaps %hu", numHeaps);

  for(i = 0; i < numHeaps; i++) {
    UInt16 heapID = MemHeapID(0, i);
    heapSize = MemHeapSize(heapID);
    MemHeapFreeBytes(heapID, &freeBytes, &maxSize);
    HostTraceOutputTL(appErrorClass, "heap %hu, size: %lu, free: %lu, max: %lu", i, heapSize, freeBytes, maxSize);
  }
}
*/

// ------------------------------------------------------------------
// Starting point of application.
// ------------------------------------------------------------------
UInt32 PilotMain(UInt16 cmd, void* cmdPBP, UInt16 launchFlags) {

  //HostTraceInit();
  //HostTraceOutputTL(appErrorClass, "start");
  //TraceMem();

  if(cmd == sysAppLaunchCmdNormalLaunch) {
    // initialize the app
    StartApplication();
    // go into the main event loop
    EventLoop();
    // clean up and exit
    StopApplication();
  }

  //TraceMem();
  //HostTraceOutputTL(appErrorClass, "stop");
  //HostTraceClose();

  return 0;
}
