/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "Stats.h"
#include "TeamStats.h"
#include "League.h"
#include "Table.h"


// ------------------------------------------------------------------
// Updates the Stats-structure with data from the supplied match.
// ------------------------------------------------------------------
void UpdateMatchData(LeagueStats* statsPtr, Match* matchPtr) {

  // result
  if(matchPtr->homeGoals > matchPtr->awayGoals) {
    statsPtr->homeWins++;
  }
  else if(matchPtr->homeGoals < matchPtr->awayGoals) {
    statsPtr->awayWins++;
  }
  else {
    statsPtr->draws++;
  }

  // goals
  statsPtr->homeGoals += matchPtr->homeGoals;
  statsPtr->awayGoals += matchPtr->awayGoals;
}


// ------------------------------------------------------------------
// Sums some data in the stats-structure
// ------------------------------------------------------------------
void SumMatchData(LeagueStats* stats) {

  stats->matchCount  = stats->homeWins + stats->draws + stats->awayWins;
  stats->goalCount   = stats->homeGoals + stats->awayGoals;
  if(stats->matchCount > 0) {
    stats->homeWinPct  = (float)stats->homeWins  / (float)stats->matchCount;
    stats->drawPct     = (float)stats->draws     / (float)stats->matchCount;
    stats->awayWinPct  = (float)stats->awayWins  / (float)stats->matchCount;
    stats->goalAvg     = (float)stats->goalCount / (float)stats->matchCount;
    stats->homeGoalAvg = (float)stats->homeGoals / (float)stats->matchCount;
    stats->awayGoalAvg = (float)stats->awayGoals / (float)stats->matchCount;
  }
  else {
    stats->homeWinPct  = 0;
    stats->drawPct     = 0;
    stats->awayWinPct  = 0;
    stats->goalAvg     = 0;
    stats->homeGoalAvg = 0;
    stats->awayGoalAvg = 0;
  }
}


// ------------------------------------------------------------------
// Update the resultArray with the match result. This function
// is used to count the number of times a certain result occur.
// ------------------------------------------------------------------
void AddMatchResult(ResultCounter* resultArray, Int16* resultCount, Match* matchPtr) {
  Int16 i;
  ResultCounter* resPtr;

  // go through all results in the array
  for(i = 0; i < *resultCount; i++) {
    resPtr = (ResultCounter*)(resultArray + (i * sizeof(ResultCounter)));
    // if we find the result for this match ->
    // increase the count for that result and exit from the loop
    if((resPtr->homeGoals == matchPtr->homeGoals) &&
       (resPtr->awayGoals == matchPtr->awayGoals))
    {
      resPtr->count++;
      break;
    }
  }

  // the result was not found -> create a new record in the array
  if(i == *resultCount) {
    resPtr = (ResultCounter*)(resultArray + (*resultCount * sizeof(ResultCounter)));
    resPtr->homeGoals = matchPtr->homeGoals;
    resPtr->awayGoals = matchPtr->awayGoals;
    resPtr->count = 1;
    (*resultCount)++;
  }

}


// ------------------------------------------------------------------
// Gets the three most frequent results from the resultArray
// and stores them in the stats-structure.
// ------------------------------------------------------------------
void SortResults(ResultCounter* resultArray, Int16 resultCount, LeagueStats* stats) {
  Int16 i;
  Int16 index0, index1, index2;
  Int16 count0, count1, count2;
  ResultCounter* resPtr;
  ResultCounter  dummy;

  // init data
  index0 = index1 = index2 = -1;
  count0 = count1 = count2 = -1;

  // loop through all the results and keep track of the
  // three with the highest count.
  for(i = 0; i < resultCount; i++) {
    resPtr = (ResultCounter*)(resultArray + (i * sizeof(ResultCounter)));
    if(resPtr->count > count2) {
      if(resPtr->count > count1) {
        if(resPtr->count > count0) {
          index2 = index1;
          count2 = count1;
          index1 = index0;
          count1 = count0;
          index0 = i;
          count0 = resPtr->count;
        }
        else {
          index2 = index1;
          count2 = count1;
          index1 = i;
          count1 = resPtr->count;
        }
      }
      else {
        index2 = i;
        count2 = resPtr->count;
      }
    }

  }

  // copy the results to the stats-structure
  dummy.count = -1;
  stats->result0 = (index0 > -1) ? *(resultArray+(index0*sizeof(ResultCounter))) : dummy;
  stats->result1 = (index1 > -1) ? *(resultArray+(index1*sizeof(ResultCounter))) : dummy;
  stats->result2 = (index2 > -1) ? *(resultArray+(index2*sizeof(ResultCounter))) : dummy;
}


// ------------------------------------------------------------------
// Returns statistics for the league
// ------------------------------------------------------------------
LeagueStats StatsGetLeagueStats(League* leaguePtr) {
  Int16          i;
  LeagueStats    stats;
  Int16          resultCount = 0;
  ResultCounter* resultArray = (ResultCounter*)MemPtrNew(sizeof(ResultCounter)*200);

  // init data
  stats.homeWins = stats.awayWins = stats.draws =
  stats.homeGoals = stats.awayGoals = 0;

  // loop through all the matches
  for(i = 0; i < leaguePtr->playedMatchCount; i++) {
    Match match = LeagueGetMatch(leaguePtr, i);
    // add data for this match
    UpdateMatchData(&stats, &match);
    // count the match results
    AddMatchResult(resultArray, &resultCount, &match);
  }

  // sum the data
  SumMatchData(&stats);
  // get the three most frequent results
  SortResults(resultArray, resultCount, &stats);

  MemPtrFree(resultArray);

  return stats;
}


// ------------------------------------------------------------------
// Returns statistics for the specified team. 'matches' is
// either 'All', 'Home', or 'Away'
// ------------------------------------------------------------------
TeamStats StatsGetTeamStats(League* leaguePtr, Int16 teamIndex, Int8 matches) {
  Int16     i;
  TeamStats stats;
  Table     table;
  TableTeam tableTeam;
  Boolean   includeHome = (matches == MATCHES_ALL || matches == MATCHES_HOME);
  Boolean   includeAway = (matches == MATCHES_ALL || matches == MATCHES_AWAY);

  // create a table
  table = TableCreate(leaguePtr, includeHome, includeAway, true, -1);

  // locate our team among the teams of the table
  i = 0;
  while(table.teams[i].teamId != teamIndex) {
    i++;
  }

  // copy data from tableteam to stats-structure
  tableTeam = table.teams[i];
  stats.tablePosition = tableTeam.position;
  stats.matchCount    = tableTeam.played;
  stats.wins          = tableTeam.won;
  stats.draws         = tableTeam.drawn;
  stats.losses        = tableTeam.lost;
  stats.goalsFor      = tableTeam.gfor;
  stats.goalsAgainst  = tableTeam.against;
  if(stats.matchCount > 0) {
    stats.winPct          = (float)stats.wins / (float)stats.matchCount;
    stats.drawPct         = (float)stats.draws / (float)stats.matchCount;
    stats.lossPct         = (float)stats.losses / (float)stats.matchCount;
    stats.goalsForAvg     = (float)stats.goalsFor / (float)stats.matchCount;
    stats.goalsAgainstAvg = (float)stats.goalsAgainst / (float)stats.matchCount;
  }
  else {
    stats.winPct          = 0;
    stats.drawPct         = 0;
    stats.lossPct         = 0;
    stats.goalsForAvg     = 0;
    stats.goalsAgainstAvg = 0;
  }

  return stats;
}
