/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "Table.h"
#include "TeamStats.h"
#include "League.h"


// ------------------------------------------------------------------
//
// TableTeam
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Initializes the values of a tableteam structure
// ------------------------------------------------------------------
void TTeamInit(TableTeam* team, Int8 teamId) {
  team->teamId   = teamId;
  team->position = 1;
  team->played = team->won = team->drawn = team->lost =
    team->gfor = team->against = team->goalDiff = team->points = 0;
}


// ------------------------------------------------------------------
// Adds a match to the specified tableteam. 'maxMatches' is maximum number of matches
// the team can have. If the team has already that number of matches this new one
// is ignored. -1 for 'maxMatches' means no limit.
// ------------------------------------------------------------------
void TTeamAddMatch(TableTeam* team, Match* match, Int16 maxMatches) {
  Int16 goalsFor;
  Int16 goalsAgainst;

  // exit function if maxMatches is already reached
  if(maxMatches > -1 && team->played >= maxMatches)
    return;

  if(match->homeTeam == team->teamId) {
    goalsFor     = match->homeGoals;
    goalsAgainst = match->awayGoals;
  }
  else {
    goalsFor     = match->awayGoals;
    goalsAgainst = match->homeGoals;
  }

  // add result (won, draw, lost)
  team->played++;
  if(goalsFor > goalsAgainst) {
    team->won++;
  }
  else if(goalsAgainst > goalsFor) {
    team->lost++;
  }
  else {
    team->drawn++;
  }

  // add goals
  team->gfor    += goalsFor;
  team->against += goalsAgainst;
}


// ------------------------------------------------------------------
// Decides what values to use when sorting the table.
// ------------------------------------------------------------------
void TTeamGetCompareValues(TableTeam* team, Int8 sortMethod, Int16 *value2, Int16 *value3, Int16 *value4) {

  switch(sortMethod) {

    case 0 : // goaldiff, goals for
      *value2 = team->goalDiff;
      *value3 = team->gfor;
      *value4 = team->won;
      break;

    case 1 : // goaldiff, wins
      *value2 = team->goalDiff;
      *value3 = team->won;
      *value4 = team->gfor;
      break;

    case 2 : // wins, goaldiff
      *value2 = team->won;
      *value3 = team->goalDiff;
      *value4 = team->gfor;
      break;

    case 3 : // wins, goals for
      *value2 = team->won;
      *value3 = team->gfor;
      *value4 = team->goalDiff;
      break;

    case 4 : // goals for, goaldiff
      *value2 = team->gfor;
      *value3 = team->goalDiff;
      *value4 = team->won;
      break;

    case 5 : // goals for, wins
      *value2 = team->gfor;
      *value3 = team->won;
      *value4 = team->goalDiff;
      break;
    }

}


// ------------------------------------------------------------------
// Compares two tableteams. The loosing team gets its position incremented.
// ------------------------------------------------------------------
void TTeamCompareTo(TableTeam* team, TableTeam* enemy, Int8 sortMethod) {

  // check points
  if(team->points < enemy->points) {
    team->position++;
  }

  // if points are equal we need to check sortMethod to see
  // what values to sort by next
  else if(team->points == enemy->points) {
    // ge the values to sort by
    Int16 teamValue2, teamValue3, teamValue4;
    Int16 enemyValue2, enemyValue3, enemyValue4;
    TTeamGetCompareValues(team, sortMethod, &teamValue2, &teamValue3, &teamValue4);
    TTeamGetCompareValues(enemy, sortMethod, &enemyValue2, &enemyValue3, &enemyValue4);

    // check the second sort value
    if(teamValue2 < enemyValue2) {
      team->position++;
    }
    // if second value is equal, check the third
    else if(teamValue2 == enemyValue2) {
      if(teamValue3 < enemyValue3) {
        team->position++;
      }
      // if third value is equal, check the fourth
      else if(teamValue3 == enemyValue3) {
        if (teamValue4 < enemyValue4) {
          team->position++;
        }
        // if the last values are equal we use the teamid
        else if (teamValue4 == enemyValue4) {
          if(team->teamId > enemy->teamId) {
            team->position++;
          }
          else {
            enemy->position++;
          }
        }
        else {
          enemy->position++;
        }
      }
      else {
        enemy->position++;
      }
    }
    else {
      enemy->position++;
    }
  }
  else {
    enemy->position++;
  }

}


// ------------------------------------------------------------------
// Sums up the points and goal difference for a team.
// ------------------------------------------------------------------
void TTeamSumUp(TableTeam* team, Int8 winPoints, Int8 bonus) {
  team->points   = (team->won * winPoints) + (team->drawn * 1) + bonus;
  team->goalDiff = team->gfor - team->against;
}



// ------------------------------------------------------------------
//
// Table
//
// ------------------------------------------------------------------

// ------------------------------------------------------------------
// Initializes a table structure.
// ------------------------------------------------------------------
void TableInit(Table* table, League *league, Boolean home, Boolean away, Boolean forward, Int16 maxMatches) {
  Int16 i;

  table->league      = league;
  table->includeHome = home;
  table->includeAway = away;
  table->forward     = forward;
  table->maxMatches  = maxMatches;

  for(i = 0; i < league->teamCount; i++) {
    TTeamInit(&table->teams[i], i);
  }
}


// ------------------------------------------------------------------
// Builds the table.
// ------------------------------------------------------------------
void TableBuild(Table* table) {
  Int16 start;
  Int16 stop;
  Int8  step;
  Int16 i;

  if(table->forward) {
    start = 0;
    stop  = table->league->playedMatchCount;
    step  = 1;
  }
  else {
    start = table->league->playedMatchCount-1;
    stop  = -1;
    step  = -1;
  }

  for(i = start; i != stop; i += step) {
    Match match = LeagueGetMatch(table->league, i);
    if(table->includeHome) {
      TTeamAddMatch(&table->teams[match.homeTeam], &match, table->maxMatches);
    }
    if(table->includeAway) {
      TTeamAddMatch(&table->teams[match.awayTeam], &match, table->maxMatches);
    }
  }

}


// ------------------------------------------------------------------
// Sorts the table.
// ------------------------------------------------------------------
void TableSort(Table* table) {
  Int16 i, j;
  Int8 teamCount = table->league->teamCount;

  // first we need to sum up all the table teams
  for(i = 0; i < teamCount; i++) {
    Int8 points;
    // get handicap/penalty points only if this is a normal, current table
    if ((table->includeHome) && (table->includeAway) && (table->maxMatches == -1)) {
      points = LeagueGetTeamPoints(table->league, i);
    }
    else {
      points = 0;
    }
    // sum up the team
    TTeamSumUp(&table->teams[i], table->league->winPoints, points);
  }

  // compare all the teams
  for(i = 0; i < teamCount; i++) {
    for(j = i+1; j < teamCount; j++) {
      TTeamCompareTo(&table->teams[i], &table->teams[j], table->league->tableSortMethod);
    }
  }

}


// ------------------------------------------------------------------
// Creates a table.
// 'home' and 'away' indicates if the table should include home and/or away matches.
// 'forward' indicates if the table should iterate the matches from the begining or the end.
// 'maxMatches' is maximum number of matches a team in the table should have.
// ------------------------------------------------------------------
Table TableCreate(League *league, Boolean home, Boolean away, Boolean forward, Int16 maxMatches) {
  Table table;

  TableInit(&table, league, home, away, forward, maxMatches);
  TableBuild(&table);
  TableSort(&table);

  return table;
}


