/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include "TeamStats.h"
#include "rcp.h"
#include "League.h"
#include "Funcs.h"
#include "Stats.h"
#include "Text.h"


// ------------------------------------------------------------------
// Constants
// ------------------------------------------------------------------
#define VISIBLE_MATCHES 11

// ------------------------------------------------------------------
// Globals declared elsewhere
// ------------------------------------------------------------------
extern AppState GState;

// ------------------------------------------------------------------
// Global variables
// ------------------------------------------------------------------
Char* GTeamNames[MAX_TEAMS];
Int16 GMatchesForTeam[MAX_TEAMS * 2];
Int8  GMatchesForTeamCount;
Int16 GSortedIndex[MAX_TEAMS];


// ------------------------------------------------------------------
// Returns the index for the first unplayed match in GMatchesForTeam
// ------------------------------------------------------------------
Int16 GetCurrentMatchIndex() {
  Int16  i;
  Match  match;
  League league = LeagueGetCurrent();

  for(i = 0; i < GMatchesForTeamCount; i++) {
    match = LeagueGetMatch(&league, GMatchesForTeam[i]);
    if(!match.isPlayed) {
      break;
    }
  }

  return i;
}


// ------------------------------------------------------------------
// Init form
// ------------------------------------------------------------------
void InitTeamForm() {
  Int8    i;
  ListPtr teamList;
  League  league = LeagueGetCurrent();
  Char*   tempPtr;

  InitTop(MODE_TEAM);

  // initialize
  for(i = 0; i < MAX_TEAMS; i++) {
    GTeamNames[i] = NULL;
  }

  // get indexes for the teams, sorted alphabetically
  LeagueGetSortedTeamIndexes(&league, GSortedIndex);

  // create an array of pointers to the team names
  for(i = 0; i < league.teamCount; i++) {
    tempPtr = LeagueGetTeam(&league, GSortedIndex[i]);
    GTeamNames[i] = MemPtrNew(11);
    StrCopy(GTeamNames[i], tempPtr);
    MemPtrUnlock(tempPtr);
  }
  // give the pointer array to the list-control
  teamList = (ListPtr)UIGetObjectPtr(popListID_team_team);
  LstSetListChoices(teamList, GTeamNames, league.teamCount);
  LstSetHeight(teamList, min(league.teamCount, 10));

  UISelectDropDown(popListID_team_team, popTrigID_team_team, GState.Team.team);
  UISelectDropDown(popListID_team_matches, popTrigID_team_matches, GState.Team.matches);
}


// ------------------------------------------------------------------
// Free form
// ------------------------------------------------------------------
void FreeTeamForm() {
  Int8 i;

  // free the pointers to the team names
  for(i = 0; i < MAX_TEAMS; i++) {
    if(GTeamNames[i]) {
      MemPtrFree(GTeamNames[i]);
      GTeamNames[i] = NULL;
    }
  }
}


// ------------------------------------------------------------------
// Updates the controls and data on the form.
// ------------------------------------------------------------------
void UpdateTeamForm(Boolean rePositionScrollBar) {
  ScrollBarPtr matchScl = (ScrollBarPtr)UIGetObjectPtr(scrollID_team_matches);
  Int16        minValue, maxValue;
  FormPtr form = FrmGetActiveForm();

  // table-view
  if(GState.Team.mode == TEAM_MODE_TABLE) {
    UISetControlLabel(form, selectTrigID_team_mode, TEXT_TEAM_TABLE);
    // hide the scrollbar by setting values to zero
    SclSetScrollBar(matchScl, 0, 0, 0, 0);
  }
  // match-view
  else {
    League league = LeagueGetCurrent();

    UISetControlLabel(form, selectTrigID_team_mode, TEXT_TEAM_MATCHES);

    // get matches to display
    GMatchesForTeamCount =
      LeagueGetMatchesForTeam(&league, GSortedIndex[GState.Team.team], GState.Team.matches, GMatchesForTeam);

    // update the scrollbar
    minValue = 0;
    maxValue = max(GMatchesForTeamCount-VISIBLE_MATCHES, 0);
    // reposition the scrollbar so the last played match is visible
    if(rePositionScrollBar) {
      GState.Team.scrollPos = GetCurrentMatchIndex() - 6;
    }
    // make sure scrollpos is sane
    GState.Team.scrollPos =
      (GState.Team.scrollPos > maxValue) ? maxValue : GState.Team.scrollPos;
    GState.Team.scrollPos =
      (GState.Team.scrollPos < minValue) ? minValue : GState.Team.scrollPos;
    // update the scrollbar
    SclSetScrollBar(matchScl, GState.Team.scrollPos, minValue, maxValue, VISIBLE_MATCHES);
  }

  // set values in drop-downs (because of plamos5 bug?)
  UISelectDropDown(popListID_team_team, popTrigID_team_team, GState.Team.team);
  UISelectDropDown(popListID_team_matches, popTrigID_team_matches, GState.Team.matches);
}


// ------------------------------------------------------------------
// Draw the table-view
// ------------------------------------------------------------------
void DrawTable() {
  Char      buf[20], buf2[10];
  Int16     x1 = 10, x2 = 100, x3 = 140;
  TeamStats stats;
  League    league = LeagueGetCurrent();

  // create the stats to display
  stats = StatsGetTeamStats(&league, GSortedIndex[GState.Team.team], GState.Team.matches);

  // display the stats
  UIDrawCharsLeft(TEXT_TEAM_POSITION, x1, 45);
  StrIToA(buf, stats.tablePosition);
  UIDrawCharsRight(buf, x2, 45);

  UIDrawCharsLeft(TEXT_TEAM_MATCHES_PLAYED, x1, 59);
  StrIToA(buf, stats.matchCount);
  UIDrawCharsRight(buf, x2, 59);

  UIDrawCharsLeft(TEXT_TEAM_WINS, x1, 71);
  StrIToA(buf, stats.wins);
  UIDrawCharsRight(buf, x2, 71);
  DoubleToPercentStr(stats.winPct, buf2, 1);
  StrPrintF(buf, "(%s)", buf2);
  UIDrawCharsRight(buf, x3, 71);

  UIDrawCharsLeft(TEXT_TEAM_DRAWS, x1, 83);
  StrIToA(buf, stats.draws);
  UIDrawCharsRight(buf, x2, 83);
  DoubleToPercentStr(stats.drawPct, buf2, 1);
  StrPrintF(buf, "(%s)", buf2);
  UIDrawCharsRight(buf, x3, 83);

  UIDrawCharsLeft(TEXT_TEAM_LOSSES, x1, 95);
  StrIToA(buf, stats.losses);
  UIDrawCharsRight(buf, x2, 95);
  DoubleToPercentStr(stats.lossPct, buf2, 1);
  StrPrintF(buf, "(%s)", buf2);
  UIDrawCharsRight(buf, x3, 95);

  UIDrawCharsLeft(TEXT_TEAM_GOALS_FOR, x1, 109);
  StrIToA(buf, stats.goalsFor);
  UIDrawCharsRight(buf, x2, 109);
  DoubleToStr(stats.goalsForAvg, buf2, 2);
  StrPrintF(buf, "(%s)", buf2);
  UIDrawCharsRight(buf, x3, 109);

  UIDrawCharsLeft(TEXT_TEAM_GOALS_AGAINST, x1, 121);
  StrIToA(buf, stats.goalsAgainst);
  UIDrawCharsRight(buf, x2, 121);
  DoubleToStr(stats.goalsAgainstAvg, buf2, 2);
  StrPrintF(buf, "(%s)", buf2);
  UIDrawCharsRight(buf, x3, 121);
}


// ------------------------------------------------------------------
// Draw the match-view
// ------------------------------------------------------------------
void DrawMatches() {
  Int16  i, y;
  Char   outBuf[20];
  Char*  outPtr;
  Int8   goalsFor, goalsAgainst;
  Char   placeChar, resultChar;
  Match  match;
  League league = LeagueGetCurrent();

  // draw the matches
  for(i = 0; i < min(GMatchesForTeamCount, VISIBLE_MATCHES); i++) {
    y = 39 + (i * 11);
    // get the match to draw
    match = LeagueGetMatch(&league, GMatchesForTeam[i + GState.Team.scrollPos]);
    // date
    StrPrintF(outBuf, TEXT_TEAM_DATE_DAY, match.dateDay);
    UIDrawCharsRight(outBuf, 16, y);
    StrPrintF(outBuf, TEXT_TEAM_DATE_MONTH, match.dateMonth);
    //StrIToA(outBuf, match.dateMonth);
    UIDrawCharsLeft(outBuf, 16, y);

    if(match.homeTeam == GSortedIndex[GState.Team.team]) {
      placeChar    = TEXT_TEAM_LETTER_HOME;
      outPtr       = LeagueGetTeam(&league, match.awayTeam);
      goalsFor     = match.homeGoals;
      goalsAgainst = match.awayGoals;
    }
    else {
      placeChar    = TEXT_TEAM_LETTER_AWAY;
      outPtr       = LeagueGetTeam(&league, match.homeTeam);
      goalsFor     = match.awayGoals;
      goalsAgainst = match.homeGoals;
    }

    // opponent
    WinDrawChars(outPtr, StrLen(outPtr), 32, y);
    MemPtrUnlock(outPtr);
    // home or away
    StrPrintF(outBuf, "(%c)", placeChar);
    WinDrawChars(outBuf, StrLen(outBuf), 95, y);

    if(match.isPlayed) {
      // result
      StrPrintF(outBuf, TEXT_TEAM_GOAL_SEP, goalsFor, goalsAgainst);
      WinDrawChars(outBuf, StrLen(outBuf), 115, y);
      // result letter
      if(goalsFor > goalsAgainst) {
        resultChar = TEXT_TEAM_LETTER_WIN;
      }
      else if(goalsFor == goalsAgainst) {
        resultChar = TEXT_TEAM_LETTER_DRAW;
      }
      else {
        resultChar = TEXT_TEAM_LETTER_LOSS;
      }
      StrPrintF(outBuf, "%c", resultChar);
      UIDrawCharsCenter(outBuf, 137, y);
    }
  }

}


// ------------------------------------------------------------------
// Redraw the form
// ------------------------------------------------------------------
void DrawTeamForm() {
  // erase the draw window (due to bug earlier OS versions)
  RectangleType eraseRect = { { 0, 39 } , { 152, 121 } };
  WinEraseRectangle(&eraseRect, 0);

  // draw the currently selected mode
  switch(GState.Team.mode) {
    case TEAM_MODE_TABLE   : DrawTable(); break;
    case TEAM_MODE_MATCHES : DrawMatches(); break;
  }
}


// ------------------------------------------------------------------
// Eventhandler for main form
// ------------------------------------------------------------------
Boolean TeamFormEvtHandler(EventPtr event) {
  Boolean handled      = false;
  Boolean updateScreen = false;
  Boolean rePositionScrollbar = false;

  switch (event->eType) {

   // form is opened
    case frmOpenEvent:
      InitTeamForm();
      updateScreen = true;
      handled = true;
      break;

   // form is closed
    case frmCloseEvent:
      FreeTeamForm();
      break;

   // form need to be updated
    case frmUpdateEvent:
      updateScreen = true;
      handled = true;
      break;

   // scrollbar is scrolled
    case sclRepeatEvent:
      GState.Team.scrollPos = event->data.sclRepeat.newValue;
      // because of a bug with redrawing scroll bars we submit a frmUpdateEvent
      // to re-draw the form instead of re-drawing it directly.
      FrmUpdateForm(formID_team, 0);
      break;

   // one of the scrollbuttons was pressed
    case keyDownEvent:
      if(GState.Team.mode == TEAM_MODE_MATCHES) {
        switch(event->data.keyDown.chr) {
          case pageUpChr :
            GState.Team.scrollPos -= VISIBLE_MATCHES;
            updateScreen = true;
            handled = true;
            break;
          case pageDownChr:
            GState.Team.scrollPos += VISIBLE_MATCHES;
            updateScreen = true;
            handled = true;
            break;
        }
      }
      break;

   // one of the mode buttons were tapped
    case ctlSelectEvent:
      switch(event->data.ctlSelect.controlID) {
        case btnID_table:
          GState.mode = MODE_TABLE;
          FrmGotoForm(formID_table);
          handled = true;
          break;
        case btnID_matches:
          GState.mode = MODE_MATCHES;
          FrmGotoForm(formID_matches);
          handled = true;
          break;
        case btnID_league:
          GState.mode = MODE_LEAGUE;
          FrmGotoForm(formID_league);
          handled = true;
          break;

        case selectTrigID_team_mode:
          GState.Team.mode = (GState.Team.mode+1) % 2;
          updateScreen = true;
          rePositionScrollbar = true;
          break;
      }
      break;

   // drop-down selected
    case popSelectEvent:
      switch(event->data.popSelect.controlID) {
        case popTrigID_league:
          //change order of leagues
          if (event->data.popSelect.selection == GState.leagueNameCount) {
            FrmPopupForm(formID_leagueOrder);
          }
          // show another league
          else if (event->data.popSelect.selection != GState.currentLeagueNameIndex) {
            ShowLeague(event->data.popSelect.selection, 0, MODE_TEAM, true, true);
          }
          break;
        case popTrigID_season:
          // show another league
          if (event->data.popSelect.selection != GState.currentSeasonIndex) {
            ShowLeague(GState.currentLeagueNameIndex, event->data.popSelect.selection, MODE_TEAM, true, false);
          }
          break;
        case popTrigID_team_team:
          GState.Team.team = event->data.popSelect.selection;
          updateScreen = true;
          rePositionScrollbar = true;
          break;
        case popTrigID_team_matches:
          GState.Team.matches = event->data.popSelect.selection;
          updateScreen = true;
          rePositionScrollbar = true;
          break;
      }
      break;


    default:

  }

 // redraw the screen if needed
  if(updateScreen) {
    UpdateTeamForm(rePositionScrollbar);
    FrmDrawForm(FrmGetActiveForm());
    DrawTeamForm();
  }

  return handled;
}
