/*
 * TeamStats by Mats Bovin (mats@mbovin.com)
 * v1.4.0 November 2002
 * Copyright (c) 2002 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.mbovin.palmos;

import java.io.*;
import java.util.*;

/**
 * This class represents a PalmOS database file (a .PDB file).
 */
public class PDBFile extends com.mbovin.palmos.Database {


    /** Constructor. */
    public PDBFile() {}

    /** Constructor. */
    public PDBFile(String name, String creator)
    {
        super(name, creator);
    }


    /** Returns the name of the database padded to 32 characters. */
    private String getPaddedName()
    {
        StringBuffer nameBuf = new StringBuffer(mName);
        nameBuf.setLength(32);
        return nameBuf.toString();
    }

    /** Writes the database to the specified stream. */
    public void writeData(DataOutputStream out) throws java.io.IOException
    {
        out.writeBytes(getPaddedName());
        out.writeShort(mAttributes);
        out.writeShort(mVersion);
        out.writeInt(mCreationDate);
        out.writeInt(mModificationDate);
        out.writeInt(mBackupDate);
        out.writeInt(mModificationNumber);
        out.writeInt(mAppInfo);
        out.writeInt(mSortInfo);
        out.writeBytes(mDBType);
        out.writeBytes(mCreator);
        out.writeInt(mUniqueIdSeed);

        out.writeInt(0); //recordlist
        out.writeShort(getRecordCount());

        // record headers
        Record rec;
        int offset = 80 + (getRecordCount() * 8);
        for (int i = 0; i < getRecordCount(); i++)
        {
            rec = getRecordAt(i);
            out.writeInt(offset); // LocalChunkID
            out.writeByte(rec.getAttributes());
            out.writeByte((byte)(i >> 16)); // uniqueID
            out.writeByte((byte)(i >> 8));  // uniqueID
            out.writeByte((byte)(i >> 0));  // uniqueID
            offset += rec.getSize();
        }

        out.writeShort(0); // pad

        // records
        Iterator iter = getRecordIterator();
        while (iter.hasNext())
        {
            rec = (Record)iter.next();
            out.write(rec.getBytes());
        }

        out.close();
    }


    /** Writes the PDB-file to disk. */
    public void saveFile(String filename) throws java.io.IOException
    {
        DataOutputStream dos =
            new DataOutputStream(new FileOutputStream(filename));
        writeData(dos);
    }


    /**
     * Reads a string of a specified length from a stream.
     * @param in the stream to read from
     * @param length the number of bytes to read
     */
    private String readPaddedString(DataInputStream in, int length)
        throws IOException
    {
        byte[] bytes = new byte[length];
        in.read(bytes);
        String name = new String(bytes).trim();
        int eos = name.indexOf('\0');
        if (eos > -1)
        {
            name = name.substring(0, eos);
        }
        return name;
    }

    /**
     * Reads the record headers.
     * @param in the stream to read from
     * @param recordCount the number of records
     */
    private List readRecordHeaders(DataInputStream in, int recordCount)
        throws java.io.IOException
    {
        Record rec;
        List recordList = new ArrayList();

        for (int i = 0; i < recordCount; i++)
        {
            int localId     = in.readInt();
            byte attrs      = in.readByte();
            byte uniqueId[] = new byte[3];
            in.read(uniqueId);
            rec = new Record(localId, attrs);
            recordList.add(rec);
        }

        return recordList;
    }


    /** Reads a database from a stream. */
    public void readData(DataInputStream in)  throws java.io.IOException
    {
        mName               = readPaddedString(in, 32);
        mAttributes         = in.readShort();
        mVersion            = in.readShort();
        mCreationDate       = in.readInt();
        mModificationDate   = in.readInt();
        mBackupDate         = in.readInt();
        mModificationNumber = in.readInt();
        mAppInfo            = in.readInt();
        mSortInfo           = in.readInt();
        mDBType             = readPaddedString(in, 4);
        mCreator            = readPaddedString(in, 4);
        mUniqueIdSeed       = in.readInt();
        mNextRecordList     = in.readInt();

        int recordCount = in.readShort();
        mRecords  = readRecordHeaders(in, recordCount);

        in.readShort(); // pad

        Record rec;
        Record nextRec;
        for (int i = 0; i < (getRecordCount() - 1); i++)
        {
            rec     = (Record)mRecords.get(i);
            nextRec = (Record)mRecords.get(i + 1);
            rec.readData(in, nextRec.getLocalId() - rec.getLocalId());
        }

        rec = (Record) mRecords.get(getRecordCount() - 1);
        rec.readData(in, in.available());

        in.close();
    }


    /**
     * Loads the PDB-file from disk.
     */
    public void loadFile(String filename) throws java.io.IOException
    {
        DataInputStream dis =
            new DataInputStream(new FileInputStream(filename));
        readData(dis);
    }

}
