/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.mbovin.ts.tools;

import java.io.*;
import java.util.*;
import java.text.*;

import com.mbovin.ts.*;


/**
 * Class for adding some extra functionality to a League object.
 */
public class LeagueSKV extends League
{

    protected int playedMatchCount_;
    protected int maxRound_;


    /**
     * Constructs a new League by reading from the
     * specified parameter file.
     */
    public LeagueSKV(String filename)
        throws java.io.IOException
    {
        Properties props = new Properties();
        props.load(new FileInputStream(filename));
        readLeague(props);
    }


    /**
     * Constructs a new League with the specified parameters.
     */
    public LeagueSKV(Properties props)
    	throws java.io.IOException
    {
        readLeague(props);
    }


    /** Loads the league the specified parameters */
    private void readLeague(Properties props)
        throws java.io.IOException
    {
        String matchFilename = props.getProperty("Matchfile");
        inStm_ = new DataInputStream(new FileInputStream(matchFilename));

        String dateChar = props.getProperty("DateChar");
        if (dateChar != null)
        {
            setDateChar(dateChar.charAt(0));
        }

        setProps(props);
        run();
    }


    /** */
    public int getMaxRound()
    {
        return maxRound_;
    }

    /**  */
    public int getPlayedMatchCount()
    {
        return playedMatchCount_;
    }


    /**
     * Returns the season of the league. The season is derived from
     * the dates of the matches. The season is returned as the year of
     * the first match. If the league is split over two years the season
     * is returned as a negative number.
     **/
    public int getSeasonAsInt()
    {
        int minYear = 9999;
        int maxYear = 0;

        GregorianCalendar calendar = new GregorianCalendar();
        Enumeration enum = matches_.elements();
        while (enum.hasMoreElements())
        {
            MatchSKV match = (MatchSKV)enum.nextElement();
            calendar.setTime(match.getRealDate());
            int year = calendar.get(Calendar.YEAR);
            if (year < minYear) minYear = year;
            if (year > maxYear) maxYear = year;
        }

        if (maxYear != minYear)
        {
            minYear = -minYear;
        }

        return minYear;
    }


    /**
     * Creates a name for the league that is used in PDB-files.
     * The name consists of the league name and the season.
     */
    public String getPDBName()
    {
        int season = getSeasonAsInt();
        int year   = Math.abs(season) % 100;

        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumIntegerDigits(2);

        String pdbName = getName() + nf.format(year);

        if (season < 0)
        {
            pdbName += nf.format((year+1) % 100);
        }

        return pdbName;
    }


    /**
     * Returns a vector with all matches. The played matches are placed
     * first followed by the not played matches. Both played and unplayed
     * matches are sorted by date. This is the order in which they are
     * represented in the PDB-file.
     */
    public Vector getMatchesPDBOrder()
    {
        Vector u = new Vector();
        for (int i = 0; i < getNumMatches(); i++)
        {
            MatchSKV m = (MatchSKV) getMatch(i);
            int j = 0;
            while (j < u.size())
            {
                MatchSKV m2 = (MatchSKV) u.elementAt(j);
                if (m.getRealDate().before(m2.getRealDate()))
                {
                    break;
                }
                j++;
            }
            u.insertElementAt(m, j);
        }

        Vector v = new Vector();
        // add played matches
        for (int i = 0; i < u.size()/*getNumMatches()*/; i++)
        {
            MatchSKV m = (MatchSKV) u.elementAt(i); //getMatch(i);
            if (m.isPlayed())
            {
                v.addElement(m);
            }
        }
        // add not played matches
        for (int i = 0; i < u.size()/*getNumMatches()*/; i++)
        {
            MatchSKV m = (MatchSKV) u.elementAt(i); //getMatch(i);
            if (!m.isPlayed())
            {
                v.addElement(m);
            }
        }

        return v;
    }


    protected boolean parseLine(String rad)
    {
        StringTokenizer tok = new StringTokenizer(rad, ";");
        int round   = Integer.parseInt(tok.nextToken());
        Date date = null;

        try
        {
            date= new SimpleDateFormat("yyyy-MM-dd").parse(tok.nextToken());
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        String home = tok.nextToken();
        String away = tok.nextToken();
        int hgoals;
        int agoals;

        try
        {
            hgoals = Integer.parseInt(tok.nextToken());
            agoals = Integer.parseInt(tok.nextToken());
        }
        catch(java.util.NoSuchElementException e)
        {
            hgoals = -1;
            agoals = -1;
        }

        if(hgoals > -1)
        {
            playedMatchCount_++;
        }
        maxRound_ = Math.max(maxRound_, round);

        // check if team already exists in league
        int homeid=-1, awayid=-1;
        int a=0;
        java.util.Enumeration enum=teams_.elements();
        while(enum.hasMoreElements() && (homeid<0 || awayid<0))
        {
            String team=((String)enum.nextElement()).trim();
            if(team.equals(home)) homeid=a;
            if(team.equals(away)) awayid=a;
            a++;
        }

        // add teams to league if they are new
        if(homeid == -1)
        {
            teams_.addElement(com.mbovin.ts.Misc.padString(home, 16));
            homeid = teams_.size()-1;
        }
        if(awayid == -1)
        {
            teams_.addElement(com.mbovin.ts.Misc.padString(away, 16));
            awayid = teams_.size()-1;
        }

        // add the match
        Match m = new Match(this, matches_.size(), homeid, awayid, hgoals, agoals, false, null);
        MatchSKV m2 = new MatchSKV(m);
        m2.setRound(round);
        m2.setRealDate(date);
        matches_.addElement(m2);
        int w;
        if((w=Integer.toString(hgoals).length()) > matchWidth_)
            matchWidth_=w;
        if((w=Integer.toString(agoals).length()) > matchWidth_)
            matchWidth_=w;

        return true;
    }

}

