/*
 * TeamStats by Mats Bovin (tspalm@mbovin.com)
 * v1.5.0 July 2003
 * Copyright (c) 2002-2003 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.mbovin.ts.tools;

import java.io.*;
import java.util.*;
import com.mbovin.palmos.*;
import com.mbovin.ts.*;


/**
 * Command line tool to create TeamStats .pdb-files from
 * a text file of match data.
 */
public class PDBCreate
{
    public static final int MAX_TEAMS       = 32;
    public static final int LEAGUENAME_SIZE = 16;
    public static final int TEAMNAME_SIZE   = 10;

    private LeagueSKV mLeague;
    private PDBFile   mPDBFile;


    /** Creates the first record, with league properties */
    private void createFirstRecord()
    {
        Record rec = new Record();
        rec.addByte(mLeague.getNumTeams());
        rec.addByte(mLeague.getMaxRound());
        rec.addShort(mLeague.getNumMatches());
        rec.addShort(mLeague.getPlayedMatchCount());
        rec.addByte(mLeague.getWinPoints());
        rec.addByte(mLeague.getTableSort());
        rec.addByte(-1); // favouriteTeam
        rec.addByte(mLeague.getLine(0)); // line 0-4
        rec.addByte(mLeague.getLine(1));
        rec.addByte(mLeague.getLine(2));
        rec.addByte(mLeague.getLine(3));
        rec.addByte(3); // dbVersion
        rec.addString(mLeague.getName(), 16);
        rec.addByte('\0');
        rec.addShort(mLeague.getSeasonAsInt());

        mPDBFile.addRecord(rec);
    }

    /** Creates the team records */
    private void createTeamRecords()
    {
        Record rec;
        for (int i = 0; i < mLeague.getNumTeams(); i++)
        {
            rec = new Record();
            rec.addString(mLeague.getTeam(i).trim(), 10);
            rec.addByte('\0');
            rec.addByte(mLeague.getBonus(mLeague.getTeam(i)));
            mPDBFile.addRecord(rec);
        }
    }

    /** Creates the round records */
    private void createRoundRecords()
    {
        // get a vector with matches which is sorted as in the pdd-file
        // played matches first, sorted by date,
        // followed by unplayed matches, sorted by date
        Vector matches = mLeague.getMatchesPDBOrder();
        Record rec;
        for (int i = 0; i < mLeague.getMaxRound(); i++)
        {
            Vector r = new Vector();
            for (int j = 0; j < mLeague.getNumMatches(); j++)
            {
                MatchSKV m = (MatchSKV)matches.elementAt(j);
                if (m.getRound() == i+1)
                {
                    // this extra loop is to make sure
                    // matches are sorted by date in the round records
                    int k = 0;
                    while (k < r.size())
                    {
                        MatchSKV m2 = (MatchSKV) matches.elementAt(((Integer)r.elementAt(k)).intValue());
                        if (m.getRealDate().before(m2.getRealDate()))
                        {
                            break;
                        }
                        k++;
                    }
                    r.insertElementAt(new Integer(j), k);
                }
            }
            rec = new Record();
            rec.addByte(r.size());
            Enumeration enum = r.elements();
            while (enum.hasMoreElements())
            {
                rec.addShort(((Integer)enum.nextElement()).intValue());
            }
            mPDBFile.addRecord(rec);
	      }
    }

    /** Creates the match records */
    private void createMatchRecords()
    {
        Record rec;
        Vector matches   = mLeague.getMatchesPDBOrder();
        Enumeration enum = matches.elements();
        while (enum.hasMoreElements())
        {
            MatchSKV m = (MatchSKV)enum.nextElement();
            rec = new Record();
            rec.addByte(m.getRound());
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.setTime(m.getRealDate());
            rec.addShort(calendar.get(Calendar.YEAR));
            rec.addByte(calendar.get(Calendar.MONTH) + 1);
            rec.addByte(calendar.get(Calendar.DATE));
            rec.addByte(m.getHomeId());
            rec.addByte(m.getAwayId());
            rec.addByte(m.getHGoals());
            rec.addByte(m.getAGoals());
            mPDBFile.addRecord(rec);
        }
    }


    /**
     * Checks that all data is correct for the league so that a
     * pdb-file can be safely created.
     */
    private boolean isLeagueValid()
    {
        // check the length of the league name
        if (mLeague.getName().length() > LEAGUENAME_SIZE)
        {
            System.out.println("Guru Meditation: Cannot create pdb-file.");
            System.out.println("The league name cannot be longer than " +
                LEAGUENAME_SIZE + " characters.");
            return false;
        }

        // make sure there's not too many teams
        if (mLeague.getNumTeams() > MAX_TEAMS)
        {
            System.out.println("Guru Meditation: Cannot create pdb-file.");
            System.out.println("The maximum number of teams in a league is " +
                MAX_TEAMS + ". This file contains " + mLeague.getNumTeams() + " teams.");
            return false;
        }

        // make sure the league contains at least one match
        if (mLeague.getNumMatches() == 0)
        {
            System.out.println("Guru Meditation: Cannot create pdb-file.");
            System.out.println("The league must contain at least one match.");
            return false;
        }

        // check the length of the team names
        // too long team names will only generate a warning, the file
        // will still be created
        for (int i = 0; i < mLeague.getNumTeams(); i++)
        {
            String teamName = mLeague.getTeam(i).trim();
            if (teamName.length() > TEAMNAME_SIZE)
            {
                System.out.println("Warning: " + teamName + " will be truncated to "
                    + teamName.substring(0, TEAMNAME_SIZE));
            }
        }

        return true;
    }


    /**
     * Reads data from the supplied argument file and creates
     * the pdb-file.
     * @return the name of the created file.
     */
    public String createDB(String argFile) throws java.io.IOException
    {
        // read the data file
        mLeague = new LeagueSKV(argFile);

        String filename = null;
        if(isLeagueValid())
        {
            // create the pdb-file object
            mPDBFile = new PDBFile(mLeague.getPDBName(), "mbTS");
            mPDBFile.setAttributes(0x0008); // backup flag

            // add all the records
            createFirstRecord();
            createTeamRecords();
            createRoundRecords();
            createMatchRecords();

            // write the file to disk
            filename = mLeague.getPDBName() + ".pdb";
            mPDBFile.saveFile(filename);
        }

        return filename;
    }


    /**
     * Starts the application.
     */
    public static void main(java.lang.String[] args)
    {
        try
        {
            System.out.println("PDBCreate v1.1.0 by Mats Bovin.");

            if (args.length < 1)
            {
                System.out.println("Guru Meditation: No parameter file specified.");
            }
            else
            {
                String filename = (new PDBCreate().createDB(args[0]));
                if (filename != null)
                {
                    System.out.println("Created " + filename + " from " + args[0]);
                }
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

}
